#******************************************************************************
# 
# File   : argos_calibrate.py
# Authors:	Clay Shepard (cws [at] rice.edu)
# License:	Copyright 2015, Rice Efficient Computing Group. All rights reserved.
#
#******************************************************************************


# todo: we need to detect clipping or too low of a signal
# todo: automatically choose ref antenna
# todo: adjust gains so that ccs don't affect dynamic range (maybe use an outside script to do this?)
# e.g. if one antenna has higher gain than other antennas, then it will have a smaller cc, which reduces it's dynamic range.  we want all ccs to be ~1.
import numpy as np
from argoscc.warpNode import *
from argoscc.argos_init import *
#import matplotlib.pyplot as plt
#from common_util import convert_from_mem, convert_to_mem
import time


def argos_calibrate(bs_nodes, mnode, ref_ant=0):
	#ref_ant = 1
	#ref_ant = 0  # 0 instead of 1 for python's sake
	write_calibration = 0
	update_pilot_length = 1
	PilotLength = 128
	PilotGuard = 80
	PilotGuardEnd = 128
	PilotTxEnd = 80
	
	#WURC changes:
	PilotLength = 256
	PilotGuard = 256
	PilotGuardEnd = 64
	PilotTxEnd = 10
	
    #NumBS = len(bs_nodes)*4  #we can make this better by counting interfaces, but for now we'll assume they all have 4...
	
	NumBS = sum([n.num_interfaces for n in bs_nodes])

	if (update_pilot_length):
		OldPilotConfig = mnode.basebandCmd('PILOT_CONFIG').result()[0]  #assume config is the same (it doesn't make sense otherwise)
		[bs_node.basebandCmd('PILOT_CONFIG', int(PilotGuard + PilotGuardEnd*2**10 + PilotTxEnd*2**20 + (np.ceil(np.log2(PilotLength))-4)*2**28) ) for bs_node in bs_nodes]
		[bs_node.basebandCmd('DISABLE_USEFFT') for bs_node in bs_nodes] #this breaks narrowband calibration
		[n.basebandCmd('NUMBS', NumBS+1) for n in bs_nodes] #should we set it back afterwards...?

	[n.basebandCmd('ENABLE_CALIB') for n in bs_nodes]  #custom function takes care of disable for us
	bs_nodes.barrier()
	mnode.basebandCmd('SINGLE_SYNC').result()
	
	ce = []
	for n in bs_nodes:  #todo: check on multiple buffer reads
		for i in range(n.num_interfaces):
			#ce[[n]*4+i,:] = bs_nodes[n].basebandCmd(n.baseband.CSI_UP_BUFF_RADIO[i], [NumBS+1], blocking = True)    
			ce.append(n.basebandCmd(n.baseband.CSI_UP_BUFF_RADIO[i], NumBS+1).result())
			#print(ce)


	ce = np.asarray(ce)
	ce = ce[:,1:len(ce)+1]
	np.fill_diagonal(ce,1)
	#print ("ce",ce)
	cc = ce/np.transpose(ce) #find calibration coefficients
	#print ("cc",cc)
	scc = cc/abs(cc)  #scale
	#print ("scc",scc)

	if(write_calibration):   
		for n in range(len(bs_nodes)):    # TODO: I don't need convert_to_mem but it would be nice to have it as a function       ######THIS SEGMENT IS NOT TESTED YET#######
			#bs_nodes[n].basebandCmd('CC_RADIO_1', convert_to_mem(scc(ref_ant,n*4-3)))
			#print("inside for loop", scc[ref_ant,(n+1)*4-4])
			bs_nodes[n].basebandCmd('CC_RADIO_1', scc[ref_ant,(n+1)*4-4])
			bs_nodes[n].basebandCmd('CC_RADIO_2', scc[ref_ant,(n+1)*4-3])
			bs_nodes[n].basebandCmd('CC_RADIO_3', scc[ref_ant,(n+1)*4-2])
			bs_nodes[n].basebandCmd('CC_RADIO_4', scc[ref_ant,(n+1)*4-1])

	[n.basebandCmd('DISABLE_CALIB') for n in bs_nodes]
	
	if (update_pilot_length):
		[n.basebandCmd('PILOT_CONFIG', OldPilotConfig) for n in bs_nodes]
	return cc, scc, ce

def test_calibration(bs_nodes, mnode, iterations=10, delay=0.0, PilotLength=256, PilotGuard=256, PilotGuardEnd=64, PilotTxEnd=10, PrintDelay=10):	

	NumBS = sum([n.num_interfaces for n in bs_nodes])	
	bs_nodes.basebandCmd('NUMBS', NumBS+1)
	ces = np.empty((iterations,NumBS,NumBS),dtype='complex64')
	ts = np.empty((iterations),dtype='float')
	
	bs_nodes.basebandCmd('PILOT_CONFIG', int(PilotGuard + PilotGuardEnd*2**10 + PilotTxEnd*2**20 + (np.ceil(np.log2(PilotLength))-4)*2**28) )
	bs_nodes.basebandCmd('ENABLE_CALIB')
	
	nodes.barrier()
	tslast = 0
	for i in range(iterations):
		mnode.basebandCmd('SINGLE_SYNC').result() #barrier
		for j,n in enumerate(bs_nodes):
			for r in range(n.num_interfaces):
				ces[i,j*4+r,:] = n.basebandCmd(n.baseband.CSI_UP_BUFF_RADIO[r], NumBS+1).result()[1:]
		ts[i] = time.time()
		time.sleep(delay)
		
		if ts[i] - ts[tslast] > PrintDelay:
			tslast = i
			print("Iteration %d out of %d, timestamp: %f (%f out of %f)" % (i,iterations,ts[i], ts[i] - ts[0], delay*iterations) )
		
	return ces,ts
	#ces,ts = test_calibration(bs_nodes,mnode,iterations=1000)
	#ces,ts = test_calibration(bs_nodes,mnode,iterations=1000,delay=1.0)
	#ces,ts = test_calibration(bs_nodes,mnode,iterations=10000,delay=4.0)
	
def plot_ccs(ces, ref_ant=0):
	import matplotlib.pyplot as plt #don't always import
	ccs = ces/np.transpose(ces, (0,2,1))
	plt.figure(figsize=(25,15))
	for i in range(ces.shape[0]):
		plt.subplot(411)
		plt.plot(np.angle(ccs[i,ref_ant,:]))
		plt.ylabel('Phase of CC')
		plt.subplot(412)
		plt.plot(abs(ccs[i,ref_ant,:]))
		plt.ylabel('Mag of CC')
		plt.subplot(413)
		plt.plot(np.angle(ces[i,ref_ant,:]))
		plt.ylabel('Phase of CE')
		plt.subplot(414)
		plt.plot(abs(ces[i,:,ref_ant]))
		plt.ylabel('Mag of CE')	

def save_ces(ces,ts,filename=None):
	import h5py
	import os
	
	if filename is None:
		if not os.path.exists('logs'):
				os.makedirs('logs')
		filename = "logs/ArgosCalibration-" + time.strftime("%Y-%m-%d-%H-%M-%S.hdf5")
	
	h5file = h5py.File(filename,'w')
	h5file.create_dataset('ChannelEstimates',data=ces)
	h5file.create_dataset('Timestamps',data=ts)
	h5file.close()
	
if __name__ == "__main__":	
	#[nodes, mnode, bs_nodes, mob_nodes, bswl_nodes, mobwl_nodes] = argos_init(nodesFile = '../matlab/config/argos.ips')
	[nodes, mnode, bs_nodes, mob_nodes, bswl_nodes, mobwl_nodes] = argos_init(nodesFile = '/usr/svn/MUBF/Code/Argos/CentralController/python/argoscc/argos.ips')
	#[nodes, mnode, bs_nodes, mob_nodes, bswl_nodes, mobwl_nodes] = argos_init(nodesFile = '../matlab/config/wurc.ips', wurc_tx_gain_db=20, wurc_rx_gain_db=0, wurc_channel=563000)
	nodes.barrier()
	print(nodes)
	#import sys	
	#sys.exit()
	#mnode.num_interfaces = 2 #temp hack -- remove #(310460496,)
	'''
	#for r in [450,490,500,550]:
	for r in [13]: #range(5,13):
		bs_nodes.interfaceCmd('rx_gains', 'RFALL_LMS6', r)
		#bs_nodes.interfaceCmd('channel', 'RFALL_LMS6', r)
		for t in [20]: #range(10,20):
			bs_nodes.interfaceCmd('tx_gains', 'RFALL_LMS6', t)
			bs_nodes.barrier()
			cc, scc, ce = argos_calibrate(bs_nodes, mnode)
			cc[1,:] = 0
			cc[:,1] = 0
			ce[1,:] = 0
			ce[:,1] = 0
			#print(ce[1][0])
			plt.figure(54)  #55?
			plt.subplot(411)
			plt.plot(np.angle(cc[0,:]))
			plt.subplot(412)
			plt.plot(abs(cc[0,:]))
			plt.subplot(413)
			plt.plot(abs(ce[0,:]))
			plt.subplot(414)
			plt.plot(abs(ce[:,0]))
			time.sleep(.05)
			bs_nodes.barrier()
			#print(np.abs(ce))
			print(np.angle(ce))
	plt.show()
	'''	
	num_samp_plot = 4096
	bs_nodes.interfaceCmd('rx_gains', 'RFALL', 0)
	for i in mnode.baseband.RXBUFF_PILOT_RADIO:
		mnode.basebandCmd(i,np.zeros(num_samp_plot))

	import matplotlib.pyplot as plt	
	
	for i in range(10):
		cc, scc, ce = argos_calibrate(bs_nodes, mnode)
		#print(ce[1][0])
		plt.figure(54)  #55?
		plt.subplot(411)
		plt.plot(np.angle(cc[0,:]))
		plt.subplot(412)
		plt.plot(abs(cc[0,:]))
		plt.subplot(413)
		plt.plot(abs(ce[0,:]))
		plt.subplot(414)
		plt.plot(abs(ce[:,0]))
		time.sleep(.05)
		bs_nodes.barrier()
		#print(np.abs(ce))
		print(np.angle(ce))
	#plt.show()
	
	
	plt.figure(55) 
	plt.subplot(411)
	plt.plot(abs(mnode.basebandCmd('RXBUFF_PILOT_RADIO_1',num_samp_plot).result()))
	plt.subplot(412)
	plt.plot(abs(mnode.basebandCmd('RXBUFF_PILOT_RADIO_2',num_samp_plot).result()))
	plt.subplot(413)
	plt.plot(abs(mnode.basebandCmd('RXBUFF_PILOT_RADIO_3',num_samp_plot).result()))
	plt.subplot(414)
	plt.plot(abs(mnode.basebandCmd('RXBUFF_PILOT_RADIO_4',num_samp_plot).result()))
	#plt.show()
'''
	for i in range(100):
		cc, scc, ce = argos_calibrate(bs_nodes, mnode)
		#plt.figure(54)       #55?
		plt.plot(np.angle(scc[0,:]))         
		#plt.show()
		#time.sleep(.1)
	plt.show()

	for i in range(100):
		cc, scc, ce = argos_calibrate(bs_nodes, mnode)
		plt.figure(54)       #55?
		plt.subplot(211)
		plt.plot(np.angle(scc[7,:]))
		plt.subplot(212)
		plt.plot(abs(cc[7,:]))
		time.sleep(.1)
	plt.show()


	x = bs_nodes[1].basebandCmd('CSI_UP_BUFF_RADIO_3', 10).result()
	x = bs_nodes[1].basebandCmd('RXBUFF_PILOT_RADIO_3', 2048).result()
	plt.figure(20)  #21?
	plt.subplot(311)
	plt.plot(abs(x))
	plt.show()
	plt.subplot(312)
	plt.plot(x.real)
	plt.show()
	plt.subplot(313)
	plt.plot(x.imag)
	plt.show()

	abs(scc)
	np.angle(ce)

	np.angle(cc)
'''
	#np.angle(cc)-np.angle(oldcc)
	#max(max(np.angle(cc)-np.angle(oldcc)))
