#******************************************************************************
# 
# File   : argos_setFrameTiming.py
# Authors:	Clay Shepard (cws [at] rice.edu)
# License:	Copyright 2015, Rice Efficient Computing Group. All rights reserved.
#
#******************************************************************************

from argoscc.warpNode import *

def argos_setFrameTiming(nodes, k=None, DownLength=None, UpLength=None, rf_path_delay=23, clock=20e6, both_mob_pilots=True, FrameLength=None, TDD_guard=None,
						 PreambleLength=272, PilotGuard=80, PilotLength=64, AGCLength=32, DCOLength=64, extra_pilots=0):

	# Timing properties that can be changed using this function:
	#   1. FrameLength
	#   2. k (number of ArgosMobiles)
	#   3. PilotGuard
	#   4. PilotLength
	#   5. PreambleLength
	#   6. DownLength
	#   7. UpLength
	#	8. TDD_guard
	#	9. CalibPilotLength
	#	10. CalibPilotGuard
	# Example function call: set_timing_values(nodes, 'FrameLength', 1000) will set
	#						frame length to 1000
	#TODO: Also have this set the ArgosMobile timing values
	#TODO:  Should we move this in to the wl_node or Argos* class?

	(bs_nodes, mob_nodes, bswl_nodes, mobwl_nodes, mnode) = sortNodes(nodes, method='type')

	# First set values to default. Perhaps read default settings off of board?
	# Probably better to set them off a script, which both this, and init.m
	# use.  And if you do it this way, you should have nodes(1), and maybe
	# even different variables for both mobile nodes and bs nodes...

	## Manually set via pilot observation
	#rf_path_delay   = 23 #77; %23 for SINGLE_SYNCs, 77 for continuous syncs	 % Extra delay to account for rf path delay. Used to align pilots %... is this assymetric? %really we should have the BS account for its tx and rx delay, and the mobile account for its tx and rx delay...
	#clock		   = 20e6
	#both_mob_pilots = 1
	#k			   = 2*len(mob_nodes); %set below
	if FrameLength is None:
		FrameLength	 = int(clock/100) #10ms
	#FrameLength	 = int(clock/100) #10ms
	#TDD_guard	   = 80+rf_path_delay #make sure to leave plenty of time for the tx to turn on
	if TDD_guard is None:
		TDD_guard	   = 80+rf_path_delay #make sure to leave plenty of time for the tx to turn on
	#PreambleLength  = 272	  
	#PilotGuard	  = 80	   # Extra pilot samples to send to avoid multipath effects and any timing issues
	#PilotLength	 = 64

	#AGCLength	   = 32
	#DCOLength	   = 64
	#extra_pilots	= 0  #extra pilots slots (e.g. to estimate noise)

	if k is None:
		k = len(mob_nodes)
		if both_mob_pilots:
			k *= 2
			
	# RF Path delay taken into account on mobile side by increasing initial frame count.
	if TDD_guard < rf_path_delay:
		raise Exception('TDD_Guard has to be longer than rf path delay %d!' % rf_path_delay)

	if np.ceil(np.log2(DCOLength)) - 5 < 0:
		raise Exception('DCOLength has to be more than 32!')

	if np.ceil(np.log2(AGCLength)) - 5 < 0:
		raise Exception('AGCLength has to be more than 32!')

	#FrameTime=FrameLength/clock
	

	# Calculate timing values to write to registers
	PreambleStart	= int(TDD_guard)
	PreambleEnd		= int(PreambleStart+PreambleLength)
	PilotStart		= int(PreambleEnd+TDD_guard) + 80 #+rf_path_delay; #we start an extra rf_path_delay later, since we'll need to subtract that from the mobile's PilotStart time
	PilotDuration	= int(PilotLength+2*PilotGuard)
	PilotEnd		= int(PilotStart+PilotDuration*(k+extra_pilots))  #extra pilot slot(s) to estimate noise)
	DownStart		= int(PilotEnd+TDD_guard)
	
	if DownLength is None:	# Write default value for downlength if it has not been specified by function args
		DownLength = int(np.floor((FrameLength-DownStart-TDD_guard)/2))
		
	#todo: find what actual max is before RSSI wrapping
	DownLength = 4096 if DownLength > 4096 else DownLength #we're having RSSI wrap, which isn't good.  let's limit this

	DownEnd = DownStart+DownLength

	UpLength = FrameLength-DownEnd-TDD_guard if UpLength is None else UpLength
	UpLength = 4096 if UpLength > 4096 else UpLength #we're having RSSI wrap, which isn't good.  let's limit this
 
	if UpLength <= 0:
		raise Exception('FrameLength too short to accomodate timing settings!')

	#Let's put the up and the end of the frame...
	UpEnd = int(FrameLength-TDD_guard-1)
	UpStart = int(UpEnd-UpLength+1)
	#UpStart = int(DownEnd + TDD_guard)
	#UpEnd = int(UpStart + UpLength - 1)

	if len(bs_nodes) > 0:
		# Write timing values to appropriate registers in all connected bs boards
		#if len(bs_nodes) != 0:
		#bs_nodes.basebandCmd('PREAMBLESTART', [PreambleStart - rf_path_delay])
		[n.basebandCmd('PreambleStart', PreambleStart - rf_path_delay) for n in bs_nodes]
		#bs_nodes.basebandCmd('PREAMBLEEND', [PreambleEnd - rf_path_delay])
		[n.basebandCmd('PREAMBLEEND', int(PreambleEnd - rf_path_delay)) for n in bs_nodes]
		if bs_nodes[0].nodetype & bs_nodes[0].WURC_TYPE: #TODO: actually set these individually for all nodes, and use the TDD_Guard to do it correctly.
			[n.basebandCmd('PILOTSTART', PilotStart - 92) for n in bs_nodes]
			[n.basebandCmd('PILOTEND', PilotEnd - 92) for n in bs_nodes]
			print('Warning! Timing set for WURC!')
		else:
			bs_nodes.basebandCmd('PILOTSTART', PilotStart)
			bs_nodes.basebandCmd('PILOTEND', PilotEnd)
			print('Warning! Timing set for Normal WARP!')
		#bs_nodes.basebandCmd('DOWNSTART', [DownStart - rf_path_delay])
		[n.basebandCmd('DOWNSTART', DownStart - rf_path_delay) for n in bs_nodes]
		#bs_nodes.basebandCmd('DOWNEND', [DownEnd - rf_path_delay])
		[n.basebandCmd('DOWNEND', DownEnd - rf_path_delay) for n in bs_nodes]
		#bs_nodes.basebandCmd('UPSTART', [UpStart])
		[n.basebandCmd('UPSTART', UpStart-20) for n in bs_nodes] #todo, fix this upstart
		#bs_nodes.basebandCmd('UPEND', [UpEnd])
		[n.basebandCmd('UPEND', UpEnd-20) for n in bs_nodes]
		#bs_nodes.basebandCmd('FRAMELENGTH', [FrameLength]) #TODO: doublecheck latency of sync -- this -3 is to account for the delay of the sync
		[n.basebandCmd('FRAMELENGTH', int(FrameLength)) for n in bs_nodes]
		#bs_nodes.basebandCmd('PILOT_CONFIG', [PilotGuard + PilotGuard*2**10 + (PilotGuard/2)*2**20 + (ceil(np.log2(PilotLength))-4)*2**28])
		[n.basebandCmd('PILOT_CONFIG', PilotGuard + PilotGuard*2**10 + int((PilotGuard/2)*2**20) + int(((np.ceil(np.log2(PilotLength)))-4)*2**28)) for n in bs_nodes]
		#bs_nodes.wl_basebandCmd('PILOT_START', PilotGuard)
		#bs_nodes.wl_basebandCmd('PILOT_LENGTH_SEL', log2(PilotLength)-4)
		#bs_nodes.wl_basebandCmd('PILOT_END', PilotGuard)
	
	
		'''
		[n.basebandCmd('PreambleStart') for n in bs_nodes]
		[n.basebandCmd('PREAMBLEEND', blocking = True) for n in bs_nodes]
		[n.basebandCmd('PILOTSTART', blocking = True) for n in bs_nodes]
		[n.basebandCmd('PILOTEND', blocking = True) for n in bs_nodes]
		[n.basebandCmd('DOWNSTART', blocking = True) for n in bs_nodes]
		[n.basebandCmd('DOWNEND', blocking = True) for n in bs_nodes]
		[n.basebandCmd('UPSTART', blocking = True) for n in bs_nodes]
		[n.basebandCmd('UPEND', blocking = True) for n in bs_nodes]		
		[n.basebandCmd('FRAMELENGTH', blocking = True) for n in bs_nodes]
		[n.basebandCmd('PILOT_CONFIG', blocking = True) for n in bs_nodes]
	'''
	else:
		print("Warning! No bs_nodes specified in argos_setFrameTiming.")
	
		
	#mob_nodes.wl_basebandCmd('PREAMBLESTART, PreambleStart);
		#print (PreambleEnd)
		#print (PilotStart - rf_path_delay)
		#print (PilotEnd - rf_path_delay)
	#if len(mob_nodes) != 0:
	#mob_nodes.basebandCmd('PREAMBLEEND', [PreambleEnd]) # + rf_path_delay);  #this is used to set the timing of the mobile up by rf_path_delay (e.g. it transmits pilots sooner)
	[mob.basebandCmd('PREAMBLEEND', PreambleEnd) for mob in mob_nodes]
	#mob_nodes.basebandCmd('PILOTSTART', [PilotStart - rf_path_delay])  #instead, let's explicitly set the PilotStart earlier, that way the Frame Counter should be the same for both the mobile and the BS (minus propagation delay)
	[mob.basebandCmd('PILOTSTART', PilotStart - rf_path_delay) for mob in mob_nodes]
	#mob_nodes.basebandCmd('PILOTEND', [PilotEnd - rf_path_delay])
	[mob.basebandCmd('PILOTEND', PilotEnd - rf_path_delay) for mob in mob_nodes]
	#mob_nodes.basebandCmd('DOWNSTART', [DownStart]) # + rf_path_delay);  #now we have to wait for the path delay, nevermind, we do this on the BS now...
	[mob.basebandCmd('DOWNSTART', DownStart) for mob in mob_nodes]

	#mob_nodes.basebandCmd('SETAGC', [DownStart + AGCLength + 16])
	[mob.basebandCmd('SETAGC', DownStart + AGCLength + 16) for mob in mob_nodes]
	#mob_nodes.basebandCmd('SETDCO', [DownStart + AGCLength + DCOLength + 16 + 16])
	[mob.basebandCmd('SETDCO', DownStart + AGCLength + DCOLength + 16 + 16) for mob in mob_nodes]
	#mob_nodes.basebandCmd('SET_DCO_LENGTH', [ceil(np.log2(DCOLength)) - 5])
	[mob.basebandCmd('SET_DCO_LENGTH', int(np.ceil(np.log2(DCOLength)) - 5)) for mob in mob_nodes]
	#mob_nodes.basebandCmd('SET_AGC_LENGTH', [ceil(np.log2(AGCLength)) - 5])
	[mob.basebandCmd('SET_AGC_LENGTH', int(np.ceil(np.log2(AGCLength)) - 5)) for mob in mob_nodes]

	#mob_nodes.basebandCmd('DOWNEND', [DownEnd]) # + rf_path_delay);
	[mob.basebandCmd('DOWNEND', DownEnd) for mob in mob_nodes]
	#mob_nodes.basebandCmd('UPSTART', [UpStart - rf_path_delay])
	[mob.basebandCmd('UPSTART', UpStart - rf_path_delay) for mob in mob_nodes]
	#mob_nodes.basebandCmd('UPEND', [UpEnd - rf_path_delay])
	[mob.basebandCmd('UPEND', UpEnd - rf_path_delay) for mob in mob_nodes]
	#mob_nodes.basebandCmd('FRAMELENGTH', [FrameLength+3]) #this +3 is to account for the latency of the sync in the base station.  better to do it here, since if it is too short in the BS, then it will break stuff.  TODO: double check
	[mob.basebandCmd('FRAMELENGTH', int(FrameLength+3)) for mob in mob_nodes]
	#mob_nodes.basebandCmd('PILOTGUARDEND', [PilotGuard+PilotLength])
	[mob.basebandCmd('PILOTGUARDEND', PilotGuard+PilotLength) for mob in mob_nodes]
	#mob_nodes.basebandCmd('PILOTGUARD', [PilotGuard])
	[mob.basebandCmd('PILOTGUARD', PilotGuard) for mob in mob_nodes]
	#mob_nodes.basebandCmd('PILOTDURATION', [PilotDuration-1])
	[mob.basebandCmd('PILOTDURATION', PilotDuration-1) for mob in mob_nodes]
