#******************************************************************************
# 
# File   : argos_writePreamblePageWeights.py
# Authors:	Clay Shepard (cws [at] rice.edu)
# License:	Copyright 2015, Rice Efficient Computing Group. All rights reserved.
#
#******************************************************************************

import numpy as np
def argos_writePreamblePageWeights(bs_nodes, preamble_weights, page_weights):
	# Writes preamble and page weights to basestation node buffers.
	# preamble/page_weights should be matrices of size MxM (M = number of BS
	# antennas). Each row of the matrices forms a beam during frame
	# transmissions.

	if(len(bs_nodes)*4 != preamble_weights.shape[1] or len(bs_nodes)*4 != page_weights.shape[1]):
		raise Exception ('argos_writePreambleWeights(): Number of preamble_weights and antennas are not equal.')

	if(preamble_weights.shape[0] > 128 or page_weights.shape[0] > 128):
		raise Exception ('argos_writePreambleWeights(): Number of preamble_weights exceed the number of memory locations in buffer.')

	preamble_weights = preamble_weights - .000001  #stupid hack because there are overflows not taken care of in buffer write.
	page_weights = page_weights - .000001  #stupid hack because there are overflows not taken care of in buffer write.
	#preamble_weights = preamble_weights / 2   #temp hack to make sure page and beacon don't clip
	#page_weights = page_weights  / 2  #
	#page_weights = np.concatenate((preamble_weights[:,5:],preamble_weights[:,:5]), axis=1) #preamble_weights.transpose() #make it sweep different.
	for i in range(len(bs_nodes)):
		#preamble_weights[:, (i+1)*4-4] = [int(round(n)) for n in preamble_weights[:, (i+1)*4-4]]  #stupid hack. Essentially, when the value is 0.99999999, it rounds down to 0 not 1 
		#preamble_weights[:, (i+1)*4-3] = [int(round(n)) for n in preamble_weights[:, (i+1)*4-3]]
		#preamble_weights[:, (i+1)*4-2] = [int(round(n)) for n in preamble_weights[:, (i+1)*4-2]]
		#preamble_weights[:, (i+1)*4-1] = [int(round(n)) for n in preamble_weights[:, (i+1)*4-1]]
		bs_nodes[i].basebandCmd('PREAMBLE_BW_BUFF_RADIO_1', preamble_weights[:, (i+1)*4-4])
		bs_nodes[i].basebandCmd('PREAMBLE_BW_BUFF_RADIO_2', preamble_weights[:, (i+1)*4-3])
		bs_nodes[i].basebandCmd('PREAMBLE_BW_BUFF_RADIO_3', preamble_weights[:, (i+1)*4-2])
		bs_nodes[i].basebandCmd('PREAMBLE_BW_BUFF_RADIO_4', preamble_weights[:, (i+1)*4-1])
	
		bs_nodes[i].basebandCmd('PAGE_BW_BUFF_RADIO_1', page_weights[:, (i+1)*4-4])
		bs_nodes[i].basebandCmd('PAGE_BW_BUFF_RADIO_2', page_weights[:, (i+1)*4-3])
		bs_nodes[i].basebandCmd('PAGE_BW_BUFF_RADIO_3', page_weights[:, (i+1)*4-2])
		bs_nodes[i].basebandCmd('PAGE_BW_BUFF_RADIO_4', page_weights[:, (i+1)*4-1])
