#******************************************************************************
# 
# File   : argosbb.py
# Authors:	Clay Shepard (cws [at] rice.edu)
# License:	Copyright 2015, Rice Efficient Computing Group. All rights reserved.
#
#******************************************************************************

#from warpNode import convert_from_mem,convert_to_mem  #this is giving me errors :/  perhaps it is a circular import?
#import warpNode #as wn
from argoscc.common_util import convert_from_mem, convert_to_mem
import struct

#there are two different flags!  One for the transport and one for the sample header.  Why is the same one being used both places?
#http://warpproject.org/trac/browser/ResearchApps/PHY/WARPLAB/WARPLab7/C_Code_Reference/include/wl_transport.h

class ArgosBB():
	"""Base class for Argos basebands (ArgosBS and ArgosMobile)."""
	REG_READ = 1       
	REG_WRITE = 2
	BUFF_READ = 3
	BUFF_WRITE = 4
	MISC_FUNC = 5
	
	Misc_Offset = 1000 
	
	@classmethod
	def procCmd(cls, cmdID, *args, mtu=1400):         #todo: use kwargs so args doesn't have to be passed as an array.
		if type(cmdID) is str:
			#todo: allow command type to be overriden, as it is in matlab.
			cmdID = getattr(cls, cmdID.upper())
		if type(cmdID) is list:   #if cmdID is a register or list of registers 
			cmdID = cmdID[0] #todo: process all of them, not just the first one.  this will involve a callback...
			#print("Warning! Currently reading multiple memories simultaneously is unsupported!")  
			if type(cmdID) is str: #it could be a list of strings, so we need to check again.
				cmdID = getattr(cls, cmdID.upper())
			
		if type(cmdID) is list:   #if cmdID is a register or list of registers 
			cmdID = cmdID[0] #todo: process all of them, not just the first one.  this will involve a callback...
			#print("Warning! Currently reading multiple memories simultaneously is unsupported!")  
		
		flag = 1		#flag is only 0 on buff read (todo: check what this actually does... I don't think it actually matters)  is this "ROBUST_FLAG"?
		# Case: BUFFER
		maxBytes = int(mtu-34-40)  #the three headers take up 34 bytes (wait, isn't it 32?  there are two missing from the diagram though), and the packet header takes 40, which leaves mtu-34-40 left.  
		maxBytes -= maxBytes%4 #make sure we have an even 4 bytes.
		if cmdID >= cls.buffer_start and cmdID  <= cls.buffer_end:
			# Read from BUFFER
			if len(args) == 1 and type(args[0]) is int:  			
				flag = 0
				cmdarg = cls.calcCmd(cls.BUFF_READ,cmdID)
				#todo:split reads in to multiple reads, then create callback to put them back together.
				#args = [0x10000000|(flag<<8),0,args[0],maxBytes,int((args[0]*4+maxBytes-1)/maxBytes)]    #(buffer selection|flags,start sample, number of samples, max number of samples in a packet, total number of packets)  #currently, the starting sample is always considered to be 0; buffer selection is 0x100000000000 (which means nothing for argos),  it also seems like there should be one last argument of IQ ID?  http://warpproject.org/trac/browser/ResearchApps/PHY/WARPLAB/WARPLab7/C_Code_Reference/wl_baseband.c
				
				#while we can send one request to get back multiple packets, for now let's just send one request per chunk, then aggregate them after they all get back.
				data_chunks = []
				samps_per_packet = maxBytes//4
				start_samp = 0;
				while start_samp+samps_per_packet < args[0]:
					data_chunks.append([0x10000000|(flag<<8),start_samp,samps_per_packet,maxBytes,1])    #(buffer selection|flags,start sample, number of samples, max number of samples in a packet, total number of packets)  #currently, the starting sample is always considered to be 0; buffer selection is 0x100000000000 (which means nothing for argos),  it also seems like there should be one last argument of IQ ID?  http://warpproject.org/trac/browser/ResearchApps/PHY/WARPLAB/WARPLab7/C_Code_Reference/wl_baseband.c
					start_samp += samps_per_packet
				data_chunks.append([0x10000000|(flag<<8),start_samp,args[0]%samps_per_packet,maxBytes,1])
					
				return cls.calcCmd(cls.BUFF_READ,cmdID), data_chunks, flag, True
			#Write to BUFFER
			else:
				#XXX: Mysterious string issue, inserting trace point
				#pdb.set_trace()
				args = convert_to_mem(args[0])
				#data_chunks = [args[i:i + maxBytes] for i in range(0, len(args), maxBytes)]
				data_chunks = []
				for i in range(0, len(args), maxBytes):  #chunk in to mtus
					data_chunks.append([0,i//4,len(args[i:i + maxBytes])//4] + list(struct.unpack(">%dI" % (len(args[i:i + maxBytes])/4), args[i:i + maxBytes]))) #chunks prepended with sample header #temporary hack to make data_chunks 4 bytes
					#data_chunks[i] = [0,maxBytes*i,maxBytes] + args[i:i + maxBytes]  #chunks prepended with sample header
				#data_chunks[0][0] = flag<<8 #we don't need this (I think it is a FLAG_LAST_WRITE, but that is only needed for r/w to DDR)
				return cls.calcCmd(cls.BUFF_WRITE,cmdID), data_chunks, flag, False
		# Case: MISC_FUNCTION
		elif cmdID > cls.Misc_Offset: 
			return cls.calcCmd(cls.MISC_FUNC,cmdID-cls.Misc_Offset), [list(args)], flag, False
		# Case: Register
		else: 
			#Reg Read
			if len(args) == 0:
				return cls.calcCmd(cls.REG_READ,cmdID), [[1]], flag, False #read one reg by default.  TODO: enable reading multiple registers  (this is kind of hard with the asyncio)
			#Reg Write
			else:
				if isinstance(args[0],complex):
					args = convert_to_mem(args)
					args = struct.unpack("%dI" % len(args)/4, args) #temporary hack to make data_chunks 4 bytes rather than 1 byte
				return cls.calcCmd(cls.REG_WRITE,cmdID), [list(args)], flag, False
				
	
	@classmethod	
	def procReply(cls, cmdID = None, data = None):
		if cmdID == cls.calcCmd(cls.BUFF_READ,cmdID & 0xFF): # ((cmdID&0xF0000) >> 16) == cls.BUFF_READ:
			return convert_from_mem(data[34:]) #buffer data starts 34 bytes in
			#cmdArgs = struct.unpack("%dh" % ( (len(data)-34)//2 ), data[34:])  #buffer data starts 34 bytes in
			#cmdArgs =(np.asarray(cmdArgs[1::2]) + np.asarray(cmdArgs[0::2])*1j)*2**-15 # convert_from_mem(cmdArgs) #convert_from_mem  #note this is a bit odd since the endian sawp....
		else:
			return struct.unpack(">%dI" % ( (len(data)-22)//4 ), data[22:])  #reg data starts 22 bytes in  #this returns a tuple.  Perhaps we should return just the value directly?
			
	@staticmethod
	def calcCmd(cmdType,cmdID):
		return (cmdType << 16) + cmdID
