#******************************************************************************
# 
# File   : common_util.py
# Authors:	Clay Shepard (cws [at] rice.edu)
# License:	Copyright 2015, Rice Efficient Computing Group. All rights reserved.
#
#******************************************************************************

import numpy as np
import struct

class CallableList(list):
	"""Nifty list that simply calls the function on every element, then returns all of their return values in a list."""
	def __init__(self, *a, **kw):
		super(CallableList, self).__init__(*a, **kw)
	def __getattr__(self, attr): #if you use __getattribute__ it overrides the list functions!
		def f(*a, **kw):
			return CallableList([getattr(n,attr)(*a, **kw) for n in self])  #since we always return futures, perhaps we should return them in a WarpNode.Futures() class?  (Actually the same thing could be accomplished by making it another CallableList...)
			#return r[0] if len(r) == 1 else r  #hrm, this is how we did the futures class...  It's probably best to always turn a list even if it is length one, just so we always use code expecting a list.  Perhaps in basebandCmd we should do this check, so that it functions normally unless it is broken in to multiple cmds.
		return f
	#def __getitem__(self, sl):  #todo have slices return callable lists
	#	return CallableList(self[sl])

#These were moved out of WarpNode to avoid circular imports.
def convert_from_mem(mem):
	'''Convert 32-bit IQ data in to NumPy array.'''
	#data = np.asarray(struct.unpack("%dh" % ( len(mem)//2 ), mem))
	data = np.fromstring(mem, dtype='i2') #use '>i2' for big-endian
	return (data[1::2] + data[0::2]*1j)*2**-15	#swap for endianness.
	
def convert_to_mem(data):
	'''Convert complex list or NumPy array in to 32-bit IQ data.'''
	data = np.asarray(data)			#Like np.array, but does not copy unless necessary.
	data = np.dot(data, 2**15)		#Generally wise to multiply with dot when mixing numpy/python datatypes -- more robust than * operator.
						#This will return a NotImplementedType rather than a cryptic failure if your ndarray contains something
						#unusual, which is a bit easier to catch / debug.
	#if len(data.shape) != 1:
	#	print('Warning! Attempting to convert non-standard array shape!')
	mem = np.ravel(np.column_stack((data.imag,data.real)))  #interleave and convert to int16. #swap for endianness.
	mem = np.clip(mem, -2**15,2**15-1).astype(np.int16, copy=False)  #round so we don't overflow and wrap.  is there a more efficient way to do this?
	return struct.pack("%dh" % len(mem), *mem) 