#******************************************************************************
# 
# File   : interface.py
# Authors:	Clay Shepard (cws [at] rice.edu)
# License:	Copyright 2015, Rice Efficient Computing Group. All rights reserved.
#
#******************************************************************************

class Interface():
	"""Base class for WARP radio interfaces."""
	#GRP = 'interface'
	_CMD_TX_EN = 1
	_CMD_RX_EN = 2
	_CMD_TX_RX_DIS = 3
	_CMD_CHANNEL = 4
	_CMD_TX_GAINS = 5
	_CMD_RX_GAINS = 6
	_CMD_TX_LPF_CORN_FREQ = 7
	_CMD_RX_LPF_CORN_FREQ = 8
	_CMD_RX_GAINCTRL_SRC = 9
	
	tx_en = 1
	rx_en = 2
	tx_rx_dis = 3
	channel = 4
	tx_gains = 5
	rx_gains = 6
	tx_lpf_corn_freq = 7
	rx_lpf_corn_freq = 8
	rx_gainctrl_src = 9
	rx_gain_mode = 9


	manual = 0

	#todo:subclass interface
	RFA = 0x10000000
	RFB = 0x20000000
	RFC = 0x30000000
	RFD = 0x40000000
	RFS = [RFA, RFB, RFC, RFD]
	RFALL = None #sum(RFS)  #Reconsider this!

	#warplab doesn't maintain state (for good reason), but let's go ahead and do it for the uplink agc...
	#tx_gain_setting = {RFA : [0,0], RFB : [0,0], RFC : [0,0], RFD : [0,0]}  #whoa, these end up being static class variables unless we initialize them!  http://stackoverflow.com/questions/11040438/class-variables-is-shared-across-all-instances-in-python
	#rx_gain_setting = {RFA : [0,0], RFB : [0,0], RFC : [0,0], RFD : [0,0]}
	
	tx_gain_setting = None
	rx_gain_setting = None

	def procCmd(node,cmdID,args=[]):
		"""Convert a command in to the proper command ID and data."""
		if type(cmdID) is str:
			cmdID = getattr(Interface,cmdID.lower())
		if cmdID == Interface.tx_gains and ( args[1] < 0 or args[1] > 3 or args[2] < 0 or args[2] > 63):
			print('Invalid TX Gains! %d, %d.' % (args[1], args[2]) )
			return
		if cmdID == Interface.rx_gains and ( args[1] < 1 or args[1] > 3 or args[2] < 0 or args[2] > 31):
			print('Invalid RX Gains! %d, %d.' % (args[1], args[2]) )
			return

		if cmdID == Interface.channel:
			if args[1] == 2.4:
				args[1] = 0
			elif args[1] == 5:
				args[1] = 1
			else:
				raise Exception ("Bad frequency chosen: expects band selection 2.4 or 5")
				return

		for i in range(len(args)):
			if type(args[i]) is str:
				if i == 0:
					args[i] = getattr(Interface,args[i].upper())
				else:
					args[i] = getattr(Interface,args[i].lower())	

		#update gains.
		if cmdID == Interface.tx_gains:
			for r in node.RFS:
				if args[0] & r:
					node.tx_gain_setting[r][:] = [args[1],args[2]] #[:] so that we don't create a new list
		if cmdID == Interface.rx_gains:
			for r in node.RFS:
				if args[0] & r:
					node.rx_gain_setting[r][:] = [args[1],args[2]]

		return cmdID, args
