#******************************************************************************
# 
# File   : interface_lms6.py
# Authors:	Clay Shepard (cws [at] rice.edu)
# License:	Copyright 2015, Rice Efficient Computing Group. All rights reserved.
#
#******************************************************************************

class InterfaceLMS6():
	"""Base class for Skylark WURC Lime LMS6002-based radio interfaces."""

	description			= 'Lime LMS6002 Interface.'
	initialize			= 1;
	wsd_serno			= 2;
	tx_en				= 3;
	rx_en				= 4;
	tx_rx_dis			= 5;
	channel				= 6;
	tx_gains			= 7;
	rx_gains			= 8;
	tx_lpf_corn_freq	= 9;
	rx_lpf_corn_freq	= 10;
	rx_gainctrl_src		= 11;
	rx_gain_mode		= 11;
	read_rssi			= 12;
	agc_state			= 13;
	agc_thresh			= 14;
	agc_target			= 15;
	agc_noise_est   	= 16;
	agc_dco_en_dis  	= 17;
	agc_trig_delay  	= 18;
	agc_reset			= 19;
	agc_done_addr   	= 20;
#	INPUT_ENABLE        = 21;
#	ENERGY_BUSY_THRESH  = 22;
#	ENERGY_AVG_LEN      = 23; *** REMOVE
#	ENERGY_BUSY_MIN_LEN = 24;
#	ENERGY_IFC_SELECTION = 25;
#	RETURN_DUMMY        =26;
#	CALIBRATION COMMANDS
	cal_loopback_en     = 27;
	cal_srcsel          = 28;
	cal_txloft          = 29;
	cal_rxdc            = 30;
	cal_tx_ssb          = 31;
	cal_rx_ssb          = 32;
	ser_cmd             = 33;

	tx_gain_max 		= 25;
	tx_gain_min 		= 0;
	rx_gain_max 		= 45;
	rx_gain_min 		= 0;
	
	manual = 0
	automatic = 1

	@classmethod	
	def procCmd(cls, cmdID, *args):
		"""Convert a command in to the proper command ID and data."""
		if type(cmdID) is str:
			cmdID = getattr(cls,cmdID.lower())
		
		args = list(args)
		
		#this is really horrible and hackish now...  I'm not sure the best way to fix it.	
		if cmdID == cls.tx_gains:
			if len(args) == 1:
				args[0] = int(args[0])  #make sure it is an int (maxim can take float dBs)
				args.append(99)
			elif len(args) > 2: #bad gain setting
				print('Invalid TX Gains! %s.' % (str(args)) )
				return

			else: #good dB gain setting
				return cls.set_tx_gain(arg[0])  
				
		if cmdID == cls.rx_gains:
			if len(args) >= 2: #bad gain setting
				print('Invalid RX Gains! %s.' % (str(args)) )
				return
			args[0] = int(args[0]) 

		#todo: probably remove this.  it just checks 'manual' vs. 'automatic', which we should check explicitly.
		for i in range(len(args)):
			if type(args[i]) is str:
				args[i] = getattr(cls,args[i].lower())

		return cmdID, args

