import numpy as np
import pandas as pd
import h5py
import matplotlib.pyplot as plt
from math import log, ceil, floor
import sys
import os
import glob


HDF5_NAME	=	'ArgosCSI-100x1-2015-08-11-15-09-02.hdf5'

#todo:make this an import from channel analysis or something cleaner

def plot_phase(CSI, timestep=.01, user=0, filename='mag.png'):
	#Plot the phase of CSI for each BS antenna for a given user and save it to a file.
	plt.figure(5,figsize=(50,60))
	plt.clf()
	for i in range(CSI.shape[2]): #plot every antenna
	#for i in range(3): #plot every antenna
		plt.plot(np.arange(0,len(CSI)*timestep,timestep),np.angle(CSI[:,user,i])+i*4)	
	plt.grid('on')
	plt.legend(loc='lower center')
	plt.xlabel('time(s)')
	plt.ylabel('Phase of each BS antenna for User %d' % user)
	plt.xlim(0,len(CSI)*timestep)
	if filename is not None and filename != '':
		plt.savefig(filename)
		
def plot_1phase(CSI, timestep=.01, user=0, filename='mag.png'):
	#Plot the phase of CSI for each BS antenna for a given user and save it to a file.
	plt.figure(5,figsize=(25,15))
	plt.clf()
	#for i in range(CSI.shape[2]): #plot every antenna
	#for i in range(3): #plot every antenna
	#	plt.plot(np.arange(0,len(CSI)*timestep,timestep),np.angle(CSI[:,user,i])+i*4)	
	plt.plot(np.arange(0,len(CSI)*timestep,timestep),np.angle(CSI[:,user]))
	plt.grid('on')
	plt.legend(loc='lower center')
	plt.xlabel('time(s)')
	plt.ylabel('Phase of each BS antenna for User %d' % user)
	plt.xlim(0,len(CSI)*timestep)
	plt.ylim(-3.15,3.15)
	if filename is not None and filename != '':
		plt.savefig(filename)

def plot_mag(CSI, timestep=.01, user=0, filename='mag.png'):
	#Plot the amplitude of CSI for each BS antenna for a given user and save it to a file.
	plt.figure(1,figsize=(25,6))
	plt.clf()
	for i in range(CSI.shape[2]): #plot every antenna
	#for i in range(3): #plot every antenna
		plt.plot(np.arange(0,len(CSI)*timestep,timestep),np.absolute(CSI[:,user,i]))	
	plt.grid('on')
	plt.legend(loc='lower center')
	plt.xlabel('time(s)')
	plt.ylabel('Absolute CSI value of each BS antenna for User %d' % user)
	plt.xlim(0,len(CSI)*timestep)
	if filename is not None and filename != '':
		plt.savefig(filename)

#hmm, may be worth checking on this: http://stackoverflow.com/questions/12323959/fast-cross-correlation-method-in-python	
def plot_corr(CSI, point=50, timestep=.01, user=0, filename='corr.png'):
	plt.figure(2,figsize=(25,6))#(70,15)) #(25,6)) (50,10))
	plt.clf()
	corr = np.empty((CSI.shape[0],))		
	for i in range(CSI.shape[0]):
		corr[i] = np.absolute(np.correlate(CSI[i,user,:],CSI[point,user,:]))
		
	plt.plot(np.arange(0,len(CSI)*timestep,timestep), corr, 'b', label='User %d' % user) #, marker='o'
	plt.plot(point*timestep,corr[point],'g', marker='o') 
	plt.xlim(0,len(CSI)*timestep)
	plt.grid('on')
	plt.legend(loc='lower center')
	plt.xlabel('Time (s)')
	plt.ylabel('Correlation')
	if filename is not None and filename != '':
		plt.savefig(filename)
		
		
def plot_corr_phase(CSI, point=50, timestep=.01, user=0, filename='corr.png'):
	plt.figure(6,figsize=(75,6))#(70,15)) #(25,6)) (50,10))
	plt.clf()
	corr = np.empty((CSI.shape[0],))		
	for i in range(CSI.shape[0]):
		corr[i] = np.angle(np.correlate(CSI[i,user,:],CSI[point,user,:]))
		
	plt.plot(np.arange(0,len(CSI)*timestep,timestep), corr, 'b', marker='o', label='User %d' % user)
	plt.plot(point*timestep,corr[point],'g', marker='o') 
	plt.xlim(0,len(CSI)*timestep)
	plt.grid('on')
	plt.legend(loc='lower center')
	plt.xlabel('Time (s)')
	plt.ylabel('Correlation')
	if filename is not None and filename != '':
		plt.savefig(filename)
		
def plot_node_corr(CSI, point=50, timestep=.01, user=0, filename='node_corr.png'):
	plt.figure(3,figsize=(40,60))
	
	plt.clf()
	corr = np.empty((CSI.shape[0],CSI.shape[2]/4))
	for n in range(int(CSI.shape[2]/4)):		
		for i in range(CSI.shape[0]):
			corr[i,n] = np.absolute(np.correlate(CSI[i,user,n:n+4],CSI[point,user,n:n+4]))	+ n/4	
		plt.plot(np.arange(0,len(CSI)*timestep,timestep), corr[:,n])
		plt.plot(point*timestep,corr[point,n],'g', marker='o') 
	plt.xlim(0,len(CSI)*timestep)
	plt.grid('on')
	plt.legend(loc='lower center')
	plt.xlabel('Time (s)')
	plt.ylabel('Correlation')
	if filename is not None and filename != '':
		plt.savefig(filename)
		
tx_bb_step = [-5, -3, -1.5, 0]
tx_rf_step = 31.0/63 #0.5
def txgain2db(bb, rf):		
	return tx_bb_step[bb] + tx_rf_step*rf
	
		
def plot_rssi(rssi, num_antennas, num_syncs, rx_rf_gain=1, avg_length=8, user=0, filename='rssi.png'):
	initial_pilot_offset = 10
	dl_buffer_length = 4096
	pilot_length = floor(dl_buffer_length/(num_antennas+1))
	##	print(pilot_length)
	##	print(rssi[2, 0, :])

	if rx_rf_gain == 1:
		rssi = np.subtract(np.multiply(70.0/(avg_length*1023), rssi), 70) # ONLY FOR MOBILE RX RF GAIN OF 1
	elif rx_rf_gain == 2:
		rssi = np.subtract(np.multiply(70.0/(avg_length*1023), rssi), 85) # ONLY FOR MOBILE RX RF GAIN OF 2
	elif rx_rf_gain == 3:
		rssi = np.subtract(np.multiply(70.0/(avg_length*1023), rssi), 100) # ONLY FOR MOBILE RX RF GAIN OF 3

##	print(rssi[1, 0, :])
	# Compute Error
	gain_err = []
	for frame_ind in range(num_syncs):
		pows = []
		for bs_ant in range(0, num_antennas+1):
			bs_ant_offset = initial_pilot_offset + bs_ant*pilot_length
##			print(bs_ant_offset)
			rssi_ind = ceil(bs_ant_offset/avg_length)+1
##			rssi_inds[bs_ant] = rssi_ind*avg_length
			# print(frame_ind)
			# print(user)
			# print(rssi_ind)
			pows.append(rssi[frame_ind][user][rssi_ind])

		sum_power = 0
		for ind_power in pows:
			sum_power += pow(10,ind_power*1.0/20)
		sum_power -= pow(10,pows[-1]*1.0/20)

		measured_rssi = pows[-1]
		gain_err.append(20*log(sum_power, 10) - measured_rssi)

	#Plot the amplitude of CSI for each BS antenna for a given user and save it to a file.
	plt.figure(10,figsize=(25,6))
	plt.clf()
	
	plt.plot(range(num_syncs), gain_err)
	
	plt.grid('on')
	plt.legend(loc='lower center')
	plt.xlabel('Frame Index')
	plt.ylabel('Gain Error for User %d' % user)
	plt.xlim(0,num_syncs)
	if filename is not None and filename != '':
		plt.savefig(filename)	
		
def plot_single_rssi(rssi, rx_rf_gain=1, avg_length=8, filename='rssi_single.png'):
	if rx_rf_gain == 1:
		rssi = np.subtract(np.multiply(70.0/(avg_length*1023), rssi), 70) # ONLY FOR MOBILE RX RF GAIN OF 1
	elif rx_rf_gain == 2:
		rssi = np.subtract(np.multiply(70.0/(avg_length*1023), rssi), 85) # ONLY FOR MOBILE RX RF GAIN OF 2
	elif rx_rf_gain == 3:
		rssi = np.subtract(np.multiply(70.0/(avg_length*1023), rssi), 100) # ONLY FOR MOBILE RX RF GAIN OF 3
		
	plt.figure(9,figsize=(25,6))
	plt.clf()
	
	plt.plot(rssi)
	
	plt.grid('on')
	plt.legend(loc='lower center')
	plt.xlabel('Sample')
	plt.ylabel('RSSI (dB)')
	#plt.xlim(0,num_syncs)
	if filename is not None and filename != '':
		plt.savefig(filename)	
#-----------------------------------------------------------------------------
# Main Function
#-----------------------------------------------------------------------------
if __name__ == '__main__':
	if len(sys.argv) >= 2:
		HDF5_NAME = str(sys.argv[1])

	newest = max(glob.iglob('*.[Hh][Dd][Ff]5'), key=os.path.getctime) #todo: test
	print(newest)
	
	f = h5py.File(HDF5_NAME,'r')	
	raw_CSI = np.array(f['CSI'])
	CSI = (raw_CSI[:,:,:,0] + raw_CSI[:,:,:,1]*1j)*2**-15 #should make this 64bit rather than 128...
	#CSI = (raw_CSI[:,:,20:52,0] + raw_CSI[:,:,20:52,1]*1j)*2**-15 #should make this 64bit rather than 128...
	RSSI = np.array(f['RSSI'])/f.attrs['pilot_length']
	if 'agc_en' in f.attrs and f.attrs['agc_en']:
		mob_tx_gains = np.array(f['mob_tx_gain'])
		#mob_tx_power = mob_tx_gains[:,:,0]*tx_rf_step + tx_bb_step[
	timestep = range(len(CSI))*f.attrs['frame_length']/1e6 #
##	frame_length is time in microseconds
	ts = f.attrs['frame_length']/1e6
##	f.close() #leave it open so we can use it from the command line
	#plot_1phase(CSI[:,:,0], user=1, timestep=ts, filename=('phase.png'))

	for u in range(CSI.shape[1]):
		print(u)
		plot_mag(CSI, user=u, timestep=ts, filename=('mag%d.png' % u))
		plot_phase(CSI, user=u, timestep=ts, filename=('phase%d.png' % u))
		plot_corr(CSI, user=u, timestep=ts, filename=('corr%d.png' % u))
		plot_corr_phase(CSI, user=u, timestep=ts, filename=('corr_phase%d.png' % u))

	u=0
	plot_node_corr(CSI, user=u, timestep=ts, filename=('node_corr-%d.png' % u))
	#plot_rssi(f['DownRSSI'], f.attrs['total_num_frames'], 1, avg_length=f.attrs['mobile_rssi_avg_length'], user=u)
	num_bs_nodes = f.attrs['num_bs_nodes']
	num_bs_antennas = 4*num_bs_nodes
	#plot_rssi(np.array(f['DownRSSI']), num_bs_antennas, f.attrs['total_num_frames'], f.attrs['mob_rx_gains'][0], f.attrs['mobile_rssi_avg_length'], u)
	#plot_single_rssi(np.array(f['DownRSSI'][90,0,1:]), f.attrs['mob_rx_gains'][0], avg_length=f.attrs['mobile_rssi_avg_length'])
##	plot_rssi(rssi, num_antennas, num_syncs, rx_rf_gain=1, avg_length=8, user=0, filename='rssi.png'):
	# f['DownRSSI'][50,0,:]
	
	
#	for i in f['FrameCompleteTime'][:,1]:
#		print(i-last)
#		last = i

#[print(e-s) for [s,e] in f['FrameCompleteTime'][:]]
