import numpy as np
import pandas as pd
import h5py
import matplotlib
#matplotlib.use('WX') #why isn't this working? #matplotlib.rcsetup.all_backends #matplotlib.get_backend()
import matplotlib.pyplot as plt
from math import log, ceil, floor
import sys
import os
import glob
import lts
import time

HDF5_NAME	=	'ArgosCSI-100x1-2015-08-11-15-09-02.hdf5'

#todo:make this an import from channel analysis or something cleaner

def plot_phase(CSI, timestep=.01, user=0, filename='mag.png'):
	#Plot the phase of CSI for each BS antenna for a given user and save it to a file.
	plt.figure(5,figsize=(50,60))
	plt.clf()
	for i in range(CSI.shape[2]): #plot every antenna
	#for i in range(3): #plot every antenna
		plt.plot(np.arange(0,len(CSI)*timestep,timestep),np.angle(CSI[:,user,i])+i*4)	
	plt.grid('on')
	plt.legend(loc='lower center')
	plt.xlabel('time(s)')
	plt.ylabel('Phase of each BS antenna for User %d' % user)
	plt.xlim(0,len(CSI)*timestep)
	if filename is not None and filename != '':
		plt.savefig(filename)
		
def plot_1phase(CSI, timestep=.01, user=0, filename='mag.png'):
	#Plot the phase of CSI for each BS antenna for a given user and save it to a file.
	plt.figure(5,figsize=(25,15))
	plt.clf()
	#for i in range(CSI.shape[2]): #plot every antenna
	#for i in range(3): #plot every antenna
	#	plt.plot(np.arange(0,len(CSI)*timestep,timestep),np.angle(CSI[:,user,i])+i*4)	
	plt.plot(np.arange(0,len(CSI)*timestep,timestep),np.angle(CSI[:,user]))
	plt.grid('on')
	plt.legend(loc='lower center')
	plt.xlabel('time(s)')
	plt.ylabel('Phase of each BS antenna for User %d' % user)
	plt.xlim(0,len(CSI)*timestep)
	plt.ylim(-3.15,3.15)
	if filename is not None and filename != '':
		plt.savefig(filename)

def plot_mag(CSI, timestep=.01, user=0, filename='mag.png'):
	#Plot the amplitude of CSI for each BS antenna for a given user and save it to a file.
	plt.figure(1,figsize=(25,6))
	plt.clf()
	for i in range(CSI.shape[2]): #plot every antenna
	#for i in range(3): #plot every antenna
		plt.plot(np.arange(0,len(CSI)*timestep,timestep),np.absolute(CSI[:,user,i]))	
	plt.grid('on')
	plt.legend(loc='lower center')
	plt.xlabel('time(s)')
	plt.ylabel('Absolute CSI value of each BS antenna for User %d' % user)
	plt.xlim(0,len(CSI)*timestep)
	if filename is not None and filename != '':
		plt.savefig(filename)

#hmm, may be worth checking on this: http://stackoverflow.com/questions/12323959/fast-cross-correlation-method-in-python	
def plot_corr(CSI, point=50, timestep=.01, user=0, filename='corr.png'):
	plt.figure(2,figsize=(25,6))#(70,15)) #(25,6)) (50,10))
	plt.clf()
	corr = np.empty((CSI.shape[0],))		
	for i in range(CSI.shape[0]):
		corr[i] = np.absolute(np.correlate(CSI[i,user,:],CSI[point,user,:]))
		
	plt.plot(np.arange(0,len(CSI)*timestep,timestep), corr, 'b', label='User %d' % user) #, marker='o'
	plt.plot(point*timestep,corr[point],'g', marker='o') 
	plt.xlim(0,len(CSI)*timestep)
	plt.grid('on')
	plt.legend(loc='lower center')
	plt.xlabel('Time (s)')
	plt.ylabel('Correlation')
	if filename is not None and filename != '':
		plt.savefig(filename)


def plot_corr_sc(CSI, point=50, timestep=.01, user=0, filename='corr_sc.png'):
	plt.figure(20+user,figsize=(250,6))#(70,15)) #(25,6)) (50,10))
	plt.clf()
	corr = np.empty((CSI.shape[0],))
	for sc in range(CSI.shape[-1]):		#range(20,25):#
		for i in range(CSI.shape[0]):
			corr[i] = np.absolute(np.correlate(CSI[i,user,0,:,sc],CSI[point,user,0,:,sc]))
			#print(i,user,CSI[i,user,0,:,sc])			
			#corr[i] = np.absolute(np.correlate(CSI[i,user,:,sc],CSI[point,user,:,sc]))  #eventually we need to parse out each user
		plt.plot(np.arange(0,len(corr)*timestep,timestep)[0:len(corr)], corr) #stupid precision makes this break sometimes unless you specify [0:len(corr)]#, 'b', label='User %d' % user) #, marker='o'
	#print(corr.shape)
	#print(corr[1:10])
	#print(np.correlate(CSI[point,0,0,:,20],CSI[point,0,0,:,20]))
	plt.plot(point*timestep,corr[point],'g', marker='o') 
	plt.xlim(0,len(CSI)*timestep)
	plt.grid('on')
	plt.legend(loc='lower center')
	plt.xlabel('Time (s)')
	plt.ylabel('Correlation of User %d' % user)
	if filename is not None and filename != '':
		plt.savefig(filename)	
		
def plot_corr_phase(CSI, point=50, timestep=.01, user=0, filename='corr.png'):
	plt.figure(6,figsize=(75,6))#(70,15)) #(25,6)) (50,10))
	plt.clf()
	corr = np.empty((CSI.shape[0],))		
	for i in range(CSI.shape[0]):
		corr[i] = np.angle(np.correlate(CSI[i,user,:],CSI[point,user,:]))
		
	plt.plot(np.arange(0,len(CSI)*timestep,timestep), corr, 'b', marker='o', label='User %d' % user)
	plt.plot(point*timestep,corr[point],'g', marker='o') 
	plt.xlim(0,len(CSI)*timestep)
	plt.grid('on')
	plt.legend(loc='lower center')
	plt.xlabel('Time (s)')
	plt.ylabel('Correlation')
	if filename is not None and filename != '':
		plt.savefig(filename)
		
def plot_node_corr(CSI, point=50, timestep=.01, user=0, filename='node_corr.png'):
	plt.figure(3,figsize=(40,60))
	
	plt.clf()
	corr = np.empty((CSI.shape[0],CSI.shape[2]/4))
	for n in range(int(CSI.shape[2]/4)):		
		for i in range(CSI.shape[0]):
			corr[i,n] = np.absolute(np.correlate(CSI[i,user,n:n+4],CSI[point,user,n:n+4]))	+ n/4	
		plt.plot(np.arange(0,len(CSI)*timestep,timestep), corr[:,n])
		plt.plot(point*timestep,corr[point,n],'g', marker='o') 
	plt.xlim(0,len(CSI)*timestep)
	plt.grid('on')
	plt.legend(loc='lower center')
	plt.xlabel('Time (s)')
	plt.ylabel('Correlation')
	if filename is not None and filename != '':
		plt.savefig(filename)
		
tx_bb_step = [-5, -3, -1.5, 0]
tx_rf_step = 31.0/63 #0.5
def txgain2db(bb, rf):		
	return tx_bb_step[bb] + tx_rf_step*rf
	
		
def plot_rssi(rssi, num_antennas, num_syncs, rx_rf_gain=1, avg_length=8, user=0, filename='rssi.png'):
	initial_pilot_offset = 10
	dl_buffer_length = 4096
	pilot_length = floor(dl_buffer_length/(num_antennas+1))
	##	print(pilot_length)
	##	print(rssi[2, 0, :])

	if rx_rf_gain == 1:
		rssi = np.subtract(np.multiply(70.0/(avg_length*1023), rssi), 70) # ONLY FOR MOBILE RX RF GAIN OF 1
	elif rx_rf_gain == 2:
		rssi = np.subtract(np.multiply(70.0/(avg_length*1023), rssi), 85) # ONLY FOR MOBILE RX RF GAIN OF 2
	elif rx_rf_gain == 3:
		rssi = np.subtract(np.multiply(70.0/(avg_length*1023), rssi), 100) # ONLY FOR MOBILE RX RF GAIN OF 3

##	print(rssi[1, 0, :])
	# Compute Error
	gain_err = []
	for frame_ind in range(num_syncs):
		pows = []
		for bs_ant in range(0, num_antennas+1):
			bs_ant_offset = initial_pilot_offset + bs_ant*pilot_length
##			print(bs_ant_offset)
			rssi_ind = ceil(bs_ant_offset/avg_length)+1
##			rssi_inds[bs_ant] = rssi_ind*avg_length
			# print(frame_ind)
			# print(user)
			# print(rssi_ind)
			pows.append(rssi[frame_ind][user][rssi_ind])

		sum_power = 0
		for ind_power in pows:
			sum_power += pow(10,ind_power*1.0/20)
		sum_power -= pow(10,pows[-1]*1.0/20)

		measured_rssi = pows[-1]
		gain_err.append(20*log(sum_power, 10) - measured_rssi)

	#Plot the amplitude of CSI for each BS antenna for a given user and save it to a file.
	plt.figure(20,figsize=(25,6))
	plt.clf()
	
	plt.plot(range(num_syncs), gain_err)
	
	plt.grid('on')
	plt.legend(loc='lower center')
	plt.xlabel('Frame Index')
	plt.ylabel('Gain Error for User %d' % user)
	plt.xlim(0,num_syncs)
	if filename is not None and filename != '':
		plt.savefig(filename)	
		
def plot_single_rssi(rssi, rx_rf_gain=1, avg_length=8, filename='rssi_single.png'):
	if rx_rf_gain == 1:
		rssi = np.subtract(np.multiply(70.0/(avg_length*1023), rssi), 70) # ONLY FOR MOBILE RX RF GAIN OF 1
	elif rx_rf_gain == 2:
		rssi = np.subtract(np.multiply(70.0/(avg_length*1023), rssi), 85) # ONLY FOR MOBILE RX RF GAIN OF 2
	elif rx_rf_gain == 3:
		rssi = np.subtract(np.multiply(70.0/(avg_length*1023), rssi), 100) # ONLY FOR MOBILE RX RF GAIN OF 3
		
	plt.figure(9,figsize=(25,6))
	plt.clf()
	
	plt.plot(rssi)
	
	plt.grid('on')
	plt.legend(loc='lower center')
	plt.xlabel('Sample')
	plt.ylabel('RSSI (dB)')
	#plt.xlim(0,num_syncs)
	if filename is not None and filename != '':
		plt.savefig(filename)	
	
	
def samps2csi(samps, num_users, samps_per_user=224, fft_size=64, offset=15+32):
	chunkstart = time.time()	
	#num_users = samps.shape[2]//samps_per_user
	usersamps = np.reshape(samps, (samps.shape[0],samps.shape[1],num_users,samps_per_user, 2) )	
	iq = np.empty((samps.shape[0],samps.shape[1],num_users,2,fft_size),dtype='complex64')
	for i in range(2): #2 seperate estimates
		iq[:,:,:,i,:] = (usersamps[:,:,:,offset+i*fft_size:offset+(i+1)*fft_size,0]+usersamps[:,:,:,offset+i*fft_size:offset+(i+1)*fft_size,1]*1j)*2**-15
	iq = iq.swapaxes(1,2)
	iq = iq.swapaxes(2,3)
	fftstart = time.time()
	csi = np.fft.fftshift(np.fft.fft(iq, fft_size, 4),4)*lts.lts_freq
	endtime = time.time()
	print("chunk time: %f fft time: %f" % (fftstart - chunkstart, endtime -fftstart) )
	csi = np.delete(csi,[0,1,2,3,4,5,32,59,60,61,62,63],4) #remove zero subcarriers
	return csi,iq	

def getCalibratedCSI(f):
	samps_per_user = f.attrs['samples_per_user']
	num_users = f.attrs['num_mob_ant']
	csi,iq = samps2csi(f['Pilot_Samples'], num_users+1, samps_per_user)
	csi_front,iq_front = samps2csi(f['AoA_Front']['Pilot_Samples'], num_users+1,samps_per_user)
	#csi_calib = csi/csi_front[csi_front.shape[0]//2,0,0,:,:]
	calib = np.mean(np.mean(csi_front[csi_front.shape[0]//2-5:csi_front.shape[0]//2+5,0,:,:,:], axis=0),axis=0)
	csi_calib = csi/calib
	
	front_calib = csi_front/calib
	"""
	plt.figure(18,figsize=(10,6))
	plt.clf()
	plt.xlabel('CSI Angle of Front Calibrated')
	plt.ylabel('Subcarrier')
	plt.plot(np.angle(front_calib[40,0,0,0:9,0]))	
	
	plt.figure(19,figsize=(10,6))
	plt.clf()
	plt.xlabel('Antenna')
	plt.ylabel('Angle')
	plt.plot(np.angle(front_calib[41,0,0,0:9,0]))	

	
	
	plt.figure(17,figsize=(10,6))
	plt.clf()
	plt.xlabel('CSI Angle of Calibration')
	plt.ylabel('Subcarrier')
	plt.plot(np.angle(calib[0,:]))	
	
	plt.figure(18,figsize=(10,6))
	plt.clf()
	plt.xlabel('CSI Angle of Front Calibrated')
	plt.ylabel('Subcarrier')
	plt.plot(np.angle(front_calib[30:40,0,0,0,:]).transpose())	
	"""
	
	return csi_calib,iq

def calCapacity(userCSI,noise,beamweights):
	"""
	Apply a set of beamweights to a set of wideband user channels and calculate the shannon capacity of the resulting channel for every sample.
	"""
	#noise = csi[:,-1,:,:,:]
	#bws = np.transpose(np.conj(csi[frame,:4,0,:,:]),(1,0,2)) for conjugate or, better: bws = np.transpose(np.conj(np.mean(csi[frame,:4,:,:,:],1)),(1,0,2))
	#userCSI = np.mean(csi[:100,:4,:,:,:],2)
	#bws = np.empty((csi.shape[3],csi.shape[1]-1,csi.shape[4]),dtype='complex64')	
	#for sc in range(csi.shape[4]):
	#	bws[:,:,sc] = np.linalg.pinv( np.mean(csi[frame,:4,:,:,sc],1) ) #* np.linalg.norm(csi[frame,:4,0,:,sc]) #either this, or the noise power has to be scaled back accordingly


	noise_bs_sc = np.mean(np.mean(np.abs(noise),0),0)  
	noise_bs_sc_scaled = noise_bs_sc * np.abs(np.sum(beamweights,1)) #should sum be outside abs?  probably not
	noise_sc = np.sum(noise_bs_sc_scaled,0)
	sig_intf = np.empty((userCSI.shape[0],userCSI.shape[1],userCSI.shape[1],userCSI.shape[3]),dtype='float32')
	for sc in range(userCSI.shape[3]):
		sig_intf[:,:,:,sc] = np.abs(np.dot(userCSI[:,:,:,sc],beamweights[:,:,sc]))
	#return sig_intf
	sig_sc = np.diagonal(sig_intf,axis1=1,axis2=2)
	sig_sc = np.swapaxes(sig_sc,1,2)
	intf_sc = np.sum(sig_intf,axis=2) - sig_sc  #todo: check that this is the right axis! #lazy, but should be quick enough (essentially we sum signal+interference, then subtract signal)
	SINR = sig_sc/(noise_sc+intf_sc)
		
	cap_sc = np.log2(1+SINR)
	cap_u = np.mean(cap_sc,axis=2)
	cap_total = np.sum(cap_u,axis=1)
	
	noise_bs_sc_scaled_u = np.swapaxes(noise_bs_sc * np.abs(np.swapaxes(beamweights,0,1)),0,1)
	noise_sc_u = np.sum(noise_bs_sc_scaled_u,0)	
	SNR = sig_sc/noise_sc_u	 #TODO, this noise is actually way too much (by roughly a factor of the number of users)
	cap_su_sc = np.log2(1+SNR)
	cap_su_u = np.mean(cap_su_sc,axis=2)
	
	return cap_total,cap_u,cap_sc,SINR,cap_su_sc,cap_su_u,SNR
	

tx_bb_step = [-5, -3, -1.5, 0]
tx_rf_step = 31/63 #0.5
def txgain2db(bb, rf):		
	return tx_bb_step[bb] + tx_rf_step*rf
#-----------------------------------------------------------------------------
# Main Function
#-----------------------------------------------------------------------------
if __name__ == '__main__':
	starttime = time.time()

	if len(sys.argv) >= 2:
		HDF5_NAME = str(sys.argv[1])
	else:
		HDF5_NAME = max(glob.iglob('*.[Hh][Dd][Ff]5'), key=os.path.getctime) #todo: test
	print(HDF5_NAME)

	frame = 10
	f = h5py.File(HDF5_NAME,'r')	
	samps = f['Pilot_Samples']
	
	samps_per_user = f.attrs['samples_per_user']
	num_users = f.attrs['num_mob_ant']
	
	plt.figure(9,figsize=(25,6))
	plt.clf()
	for u in range(samps.shape[1]):
		plt.plot(samps[frame,u,:,0])

	'''
	iq = (samps[:,:,:,0]+samps[:,:,:,1]*1j)*2**-15
	max_iq = np.max(np.abs(iq), axis=2)
	plt.figure(90,figsize=(25,6))
	plt.clf()
	for i in range(samps.shape[1]):
		plt.plot(max_iq[:,i])
	'''
	print(time.time()-starttime)
	#sys.exit()	
	
	#plt.plot(np.angle((np.fft.fftshift(np.fft.fft(np.fft.ifft(np.fft.ifftshift(lts.lts_freq))))*lts.lts_freq)))
	iq = (samps[frame,0,0:samps_per_user,0]+samps[frame,0,0:samps_per_user,1]*1j)*2**-15
	#print(lts.findLTS(iq))

	offset = lts.findLTS(iq)+32
	print(offset)	
	#iq = (samps[frame,0,samps_per_user*3:4*samps_per_user,0]+samps[frame,0,samps_per_user*3:4*samps_per_user,1]*1j)*2**-15
	#print(lts.findLTS(iq))
	
	#offset = lts.findLTS(iq)+32
	#print(offset)
	
	

	#sys.exit()
	#mod = np.ones(64)
	#mod[::2] = -1 #I have no idea why we need to do this...
	"""	
	est1 = np.fft.fftshift(np.fft.fft(iq[offset:offset+64]))*lts.lts_freq#*mod
	est2 = np.fft.fftshift(np.fft.fft(iq[offset+64:offset+128]))*lts.lts_freq#*mod
	plt.figure(10,figsize=(10,6))
	plt.clf()
	plt.plot(np.abs(est1))
	plt.plot(np.abs(est2))
	
	plt.figure(11,figsize=(10,6))
	plt.clf()
	plt.plot(np.angle(est1))
	plt.plot(np.angle(est2))
	"""
	
	#plt.figure(12,figsize=(10,6))
	#plt.clf()
	#plt.plot(np.angle(lts.lts_freq))
	csistart = time.time()
	#csi,iq = samps2csi(f['Pilot_Samples'][::100], f.attrs['num_mob_ant']+1, samps_per_user=samps_per_user) #, fft_size=int(64*f.attrs['uplink_sample_rate']), offset=int(47*f.attrs['uplink_sample_rate'])) #temporary hack on fft size
	csi,iq = samps2csi(f['Pilot_Samples'], f.attrs['num_mob_ant']+1, samps_per_user=samps_per_user) #, fft_size=int(64*f.attrs['uplink_sample_rate']), offset=int(47*f.attrs['uplink_sample_rate'])) #temporary hack on fft size
	#csi,iq = getCalibratedCSI(f)
	csiend = time.time()

	'''
	noise = csi[:,-1,:,:,:]
	conjbws = np.transpose(np.conj(np.mean(csi[frame,:-1,:,:,:],1)),(1,0,2))
	zfbws = np.empty((csi.shape[3],csi.shape[1]-1,csi.shape[4]),dtype='complex64')	
	for sc in range(csi.shape[4]):
		zfbws[:,:,sc] = np.linalg.pinv( np.mean(csi[frame,:-1,:,:,sc],1) )

	userCSI = np.mean(csi[:,:-1,:,:,:],2)

	conjcap_total,conjcap_u,conjcap_sc,conjSINR,conjcap_su_sc,conjcap_su_u,conjSNR = calCapacity(userCSI,noise,conjbws)
	zfcap_total,zfcap_u,zfcap_sc,zfSINR,zfcap_su_sc,zfcap_su_u,zfSNR = calCapacity(userCSI,noise,zfbws)
	plt.figure(1000, figsize=(50,10))
	plt.plot(conjcap_u)
	plt.figure(1001, figsize=(50,10))
	plt.plot(zfcap_u)	
	
	sys.exit()
	'''

	max_iq = np.max(np.max(np.abs(iq), axis=4), axis=2)
	
	for u in range(f.attrs['num_mob_ant']+1):		
		plt.figure(60+u,figsize=(10,6))
		plt.clf()
		plt.ylabel('Max IQ of User %d' % u)
		plt.xlabel('Frame')
		for bs in range(max_iq.shape[2]):
			plt.plot(max_iq[:,u,bs])

	
	if 'mob_tx_gain' in f:		
		for u in range(f.attrs['num_mob_ant']):		
			plt.figure(600+u,figsize=(10,6))
			plt.clf()
			plt.ylabel('Gain of User %d' % u)
			plt.xlabel('Frame')
			g = []
			for i in range(f['mob_tx_gain'].shape[0]):
				#g.append(txgain2db(*f['mob_tx_gain'][i,u,:]))
				g.append(f['mob_tx_gain'][i,u,:])
			plt.plot(g)
	'''		
	for u in range(f.attrs['num_mob_ant']+1):	
		plt.figure(70+u,figsize=(10,6))
		plt.clf()
		plt.ylabel('RSSI of User %d' % u)
		plt.xlabel('Frame')
		for i in range(f['RSSI'].shape[2]):
			plt.plot(f['RSSI'][:,u,i])
	
	corrtime = time.time()
	
	for u in range(f.attrs['num_mob_ant']):
		plot_corr_sc(csi, user=u, timestep=f.attrs['frame_length']/20e6)
	endtime = time.time()
	print("Total time: %f  corr time: %f csitime: %f" % (endtime - starttime, endtime - corrtime, csiend-csistart))
	'''	
	
	plot_node_corr(csi, user=u, timestep=f.attrs['frame_length']/20e6)

		
	'''
	for u in range(f.attrs['num_mob_ant']+1):
		plt.figure(10+u,figsize=(10,6))
		plt.clf()
		plt.xlabel('CSI Angle of User %d for Frame %d' % (u,frame))
		plt.ylabel('Subcarrier')
		plt.plot(np.angle(csi[frame,u,0,0,:]))
		plt.plot(np.angle(csi[frame,u,1,0,:]))
		
	for u in range(f.attrs['num_mob_ant']+1):
		plt.figure(10+u,figsize=(10,6))
		plt.clf()
		plt.xlabel('CSI Angle of User %d for Frame %d' % (u,frame))
		plt.ylabel('Subcarrier')
		plt.plot(np.abs(csi[frame,u,0,0,:]))
		plt.plot(np.abs(csi[frame,u,1,0,:]))		
	'''
	if 'UplinkData' in f:
		plt.figure(52)
		for bs in range(f.attrs['num_bs_ant']):
			plt.plot(f['UplinkData'][frame,bs,:,0])	
	plt.show()	
	#time.sleep(10)
	sys.exit()			
	for u in range(f.attrs['num_mob_ant']+1):
		plt.figure(80+u)
		plt.clf()
		plt.xlabel('Antenna')
		plt.ylabel('RSSI of User %d for frame %d' % (u,frame))
		plt.plot(f['RSSI'][frame,u,:])
		#plt.plot(max_iq[frame,u,:])
		
	for u in range(f.attrs['num_mob_ant']+1):
		plt.figure(90+u,figsize=(100,6))
		plt.clf()
		plt.xlabel('Antenna')
		plt.ylabel('Max IQ of User %d for frame %d' % (u,frame))
		#plt.plot(f['RSSI'][frame,u,:])
		plt.plot(max_iq[frame,u,:])
	"""
	for u in range(f.attrs['num_mob_ant']+1):
		plt.figure(50+u,figsize=(10,6))
		plt.clf()
		plt.xlabel('CSI Magnitude of User %d for Frame %d' % (u,frame))
		plt.ylabel('Subcarrier')
		plt.plot(np.abs(csi[frame,u,0,0,:]))
		plt.plot(np.abs(csi[frame,u,1,0,:]))
	

	for frame in range(300,500,10):
		plt.figure(200+frame,figsize=(10,6))
		plt.clf()
		plt.xlabel('spatial fft Frame %d' % (frame))
		plt.ylabel('magnitude')
		plt.plot(np.abs(np.fft.fft(csi[frame,0,0,0:9,20])))
	
	for frame in range(300,500,10):
		plt.figure(300+frame,figsize=(10,6))
		plt.clf()
		plt.xlabel('Antenna')
		plt.ylabel('Angle')
		plt.plot(np.unwrap(np.angle(csi[frame,0,0,0:9,20])))
		plt.plot(np.unwrap(np.angle(csi[frame,0,0,0:9,21])))
		"""
	slope = np.empty((csi.shape[0]))

	for frame in range(csi.shape[0]):
		slope[frame] = -np.mean(np.diff(np.unwrap(np.angle(csi[frame,0,0,0:9,25]))))
	plt.figure(555,figsize=(10,6))
	plt.plot(slope)
	
	slope = np.empty((csi.shape[0]))
	for frame in range(csi.shape[0]):
		slope[frame] = -np.mean(np.diff(np.unwrap(np.angle(csi[frame,3,0,0:9,25]))))
	plt.figure(557,figsize=(10,6))
	plt.plot(slope)
	
	slope = np.empty((22,csi.shape[0],))
	for sc in range(8,30):
		for frame in range(csi.shape[0]):
			slope[sc-8,frame] = -np.mean(np.diff(np.unwrap(np.angle(csi[frame,0,0,0:9,sc]))))
	plt.figure(556,figsize=(10,6))
	plt.plot(slope.transpose())

	csi_left,iq_left = samps2csi(f['AoA_Left']['Pilot_Samples'], num_users+1,samps_per_user)	
	csi_right,iq_right = samps2csi(f['AoA_Right']['Pilot_Samples'], num_users+1,samps_per_user)	
	csi_front,iq_front = samps2csi(f['AoA_Front']['Pilot_Samples'], num_users+1,samps_per_user)	
	
	slope = np.empty((22,csi_left.shape[0],))
	for sc in range(8,30):
		for frame in range(csi_left.shape[0]):
			slope[sc-8,frame] = -np.mean(np.diff(np.unwrap(np.angle(csi_left[frame,0,0,0:9,sc]))))
	plt.figure(656,figsize=(10,6))
	plt.xlabel('left')
	plt.plot(slope.transpose())
	slope = np.empty((22,csi_right.shape[0],))
	for sc in range(8,30):
		for frame in range(csi_right.shape[0]):
			slope[sc-8,frame] = -np.mean(np.diff(np.unwrap(np.angle(csi_right[frame,0,0,0:9,sc]))))
	plt.figure(756,figsize=(10,6))
	plt.xlabel('right')
	plt.plot(slope.transpose())
	slope = np.empty((22,csi_front.shape[0],))
	for sc in range(8,30):
		for frame in range(csi_front.shape[0]):
			slope[sc-8,frame] = -np.mean(np.diff(np.unwrap(np.angle(csi_front[frame,0,0,0:9,sc]))))
	plt.figure(856,figsize=(10,6))
	plt.xlabel('front')
	plt.plot(slope.transpose())
		
	#plt.figure(14,figsize=(10,6))
	#plt.clf()
	#plt.plot(np.angle(csi[frame,1,0,0,:]))
	#plt.plot(np.angle(csi[frame,1,1,0,:]))
	
	#u=0
	#sc=20
	#plot_corr(csi1[:,:,sc], user=u)


	#for u in range(f.attrs['num_mob_ant']+1):
	#for u in [0]:

	
	corrtime = time.time()
	for u in range(f.attrs['num_mob_ant']+1):

		plot_corr_sc(csi, user=u, timestep=f.attrs['frame_length']/20e6)
	endtime = time.time()
	print("Total time: %f  corr time: %f csitime: %f" % (endtime - starttime, endtime - corrtime, csiend-csistart))
	
	'''
	raw_CSI = np.array(f['CSI'])
	CSI = (raw_CSI[:,:,:,0] + raw_CSI[:,:,:,1]*1j)*2**-15 #should make this 64bit rather than 128...
	#CSI = (raw_CSI[:,:,20:52,0] + raw_CSI[:,:,20:52,1]*1j)*2**-15 #should make this 64bit rather than 128...
	RSSI = np.array(f['RSSI'])/f.attrs['pilot_length']
	if 'agc_en' in f.attrs and f.attrs['agc_en']:
		mob_tx_gains = np.array(f['mob_tx_gain'])
		#mob_tx_power = mob_tx_gains[:,:,0]*tx_rf_step + tx_bb_step[
	timestep = range(len(CSI))*f.attrs['frame_length']/1e6 #
##	frame_length is time in microseconds
	ts = f.attrs['frame_length']/1e6
##	f.close() #leave it open so we can use it from the command line
	#plot_1phase(CSI[:,:,0], user=1, timestep=ts, filename=('phase.png'))

	for u in range(CSI.shape[1]):
		print(u)
		plot_mag(CSI, user=u, timestep=ts, filename=('mag%d.png' % u))
		plot_phase(CSI, user=u, timestep=ts, filename=('phase%d.png' % u))
		plot_corr(CSI, user=u, timestep=ts, filename=('corr%d.png' % u))
		plot_corr_phase(CSI, user=u, timestep=ts, filename=('corr_phase%d.png' % u))

	u=0
	plot_node_corr(CSI, user=u, timestep=ts, filename=('node_corr-%d.png' % u))
	#plot_rssi(f['DownRSSI'], f.attrs['total_num_frames'], 1, avg_length=f.attrs['mobile_rssi_avg_length'], user=u)
	num_bs_nodes = f.attrs['num_bs_nodes']
	num_bs_antennas = 4*num_bs_nodes
	'''
	#plot_rssi(np.array(f['DownRSSI']), num_bs_antennas, f.attrs['total_num_frames'], f.attrs['mob_rx_gains'][0], f.attrs['mobile_rssi_avg_length'], u)
	#plot_single_rssi(np.array(f['DownRSSI'][90,0,1:]), f.attrs['mob_rx_gains'][0], avg_length=f.attrs['mobile_rssi_avg_length'])
##	plot_rssi(rssi, num_antennas, num_syncs, rx_rf_gain=1, avg_length=8, user=0, filename='rssi.png'):
	# f['DownRSSI'][50,0,:]
	
	
#	for i in f['FrameCompleteTime'][:,1]:
#		print(i-last)
#		last = i

#[print(e-s) for [s,e] in f['FrameCompleteTime'][:]]
