import numpy as np
import pandas as pd
import h5py
import matplotlib
import matplotlib.pyplot as plt
from math import log, ceil, floor
import sys
import os
import glob
import lts
import time
import scipy.io as sio


def samps2csi_large(samps, num_users, samps_per_user=224, offset=47, chunk_size=1000):
	"""Wrapper function for samps2csi_main for to speed up large logs by leveraging data-locality. Chunk_size may need to be adjusted based on your computer."""
	
	if samps.shape[0]> chunk_size:
		#rather than memmap let's just increase swap... should be just as fast.
		#csi = np.memmap(os.path.join(_here,'temp1.mymemmap'), dtype='complex64', mode='w+', shape=(samps.shape[0], num_users, 2, samps.shape[1],52))
		#iq = np.memmap(os.path.join(_here,'temp2.mymemmap'), dtype='complex64', mode='w+', shape=(samps.shape[0], num_users, 2, samps.shape[1],64))
		csi = np.empty((samps.shape[0], num_users, 2, samps.shape[1],52), dtype='complex64')
		iq = np.empty((samps.shape[0], num_users, 2, samps.shape[1],64), dtype='complex64')
		chunk_num = samps.shape[0]//chunk_size
		for i in range(chunk_num):
			csi[i*chunk_size:i*chunk_size+chunk_size], iq[i*chunk_size:i*chunk_size+chunk_size] = samps2csi(samps[i*chunk_size:(i*chunk_size+chunk_size),:,:,:], num_users, samps_per_user=samps_per_user)
		csi[chunk_num*chunk_size:],iq[chunk_num*chunk_size:] = samps2csi(samps[chunk_num*chunk_size:,:,:,:], num_users, samps_per_user=samps_per_user)
	else:
		csi,iq = samps2csi(samps, num_users, samps_per_user=samps_per_user, offset=offset)
	return csi,iq


def samps2csi(samps, num_users, samps_per_user=224, fft_size=64, offset=15+32):
	chunkstart = time.time()
	#num_users = samps.shape[2]//samps_per_user
	usersamps = np.reshape(samps, (samps.shape[0],samps.shape[1],num_users,samps_per_user, 2) )
	iq = np.empty((samps.shape[0],samps.shape[1],num_users,2,fft_size),dtype='complex64')
	for i in range(2): #2 seperate estimates
		iq[:,:,:,i,:] = (usersamps[:,:,:,offset+i*fft_size:offset+(i+1)*fft_size,0]+usersamps[:,:,:,offset+i*fft_size:offset+(i+1)*fft_size,1]*1j)*2**-15
	iq = iq.swapaxes(1,2)
	iq = iq.swapaxes(2,3)
	fftstart = time.time()
	csi = np.fft.fftshift(np.fft.fft(iq, fft_size, 4),4)*lts.lts_freq
	endtime = time.time()
	print("chunk time: %f fft time: %f" % (fftstart - chunkstart, endtime -fftstart) )
	csi = np.delete(csi,[0,1,2,3,4,5,32,59,60,61,62,63],4) #remove zero subcarriers
	return csi,iq


#-----------------------------------------------------------------------------
# Main Function
#-----------------------------------------------------------------------------
if __name__ == '__main__':
	starttime = time.time()
	ue_index = 0
	cal_index = 3
	carr_index = 13
	loc_name = 'nlos_loc16'
	HDF5_NAME = 'ch1_' + loc_name + '.hdf5'
	#HDF5_NAME = max(glob.iglob('*.[Hh][Dd][Ff]5'), key=os.path.getctime) #todo: test
	print(HDF5_NAME)
    
	frame = 100
	f = h5py.File(HDF5_NAME,'r')
	samps = f['Pilot_Samples']

	samps_per_user = f.attrs['samples_per_user']
	num_users = f.attrs['num_mob_ant']

	plt.figure(1,figsize=(25,6))
	plt.clf()
	for u in range(samps.shape[1]): # u is antenna element
		plt.plot(samps[frame,u,:,0])  #plot i/q samples of each antenna at
	#plt.savefig('iq_ch1_los_loc1.pdf')
	print(time.time()-starttime)

	csistart = time.time()
	#csi,iq = samps2csi(f['Pilot_Samples'][::100], f.attrs['num_mob_ant']+1, samps_per_user=samps_per_user) #, fft_size=int(64*f.attrs['uplink_sample_rate']), offset=int(47*f.attrs['uplink_sample_rate'])) #temporary hack on fft size
	csi,iq = samps2csi_large(f['Pilot_Samples'], f.attrs['num_mob_ant']+1, samps_per_user=samps_per_user) #, fft_size=int(64*f.attrs['uplink_sample_rate']), offset=int(47*f.attrs['uplink_sample_rate'])) #temporary hack on fft size
	csiend = time.time()

	#noise = csi[:,-1,:,:,:]  #noise is last set of data.
	#noiseA = np.mean(np.abs(noise),1)
	#userCSI = np.mean(csi[:,:num_users,:,:,:],2) #don't include noise, average over both LTSs
	#userCSI = np.mean(csi[:,:,:,:,:],2) #Frame, User, BS ant, Subcarrier
	userCSI = csi[:,:,0,:,:] #Frame, User, BS ant, Subcarrier
	#sio.savemat('ch1_NLOS_loc3.mat',mdict={'ch1_NLOS_loc3':userCSI})

	frame_num = userCSI.shape[0]
	frame_rem = 10
	noiseP = np.empty((frame_num - frame_rem,))
	chalP = np.empty((frame_num - frame_rem,))
	corr = np.empty((frame_num - frame_rem,))
	#carr_index = 0
	chal1 = userCSI[frame_rem,ue_index,4:68,carr_index]
	for i in range(frame_num - frame_rem):
		noise = userCSI[frame_rem + i,4,4:68,carr_index]
		noiseP[i] = np.mean(np.square(np.abs(noise)))
		chal = userCSI[frame_rem + i,ue_index,4:68,carr_index]
		chalP[i] = np.mean(np.square(np.abs(chal)))
		corr[i] = np.absolute(np.correlate(chal,chal1)/(np.linalg.norm(chal) * np.linalg.norm(chal1)))


	plt.figure(2)
	plt.clf()
	plt.plot(corr)
	plt.xlabel('Frame Index')
	plt.ylabel('Correlation')
	#plt.savefig('cor_ch14_LOS_loc1.pdf')

	plt.figure(3)
	plt.clf()
	plt.plot(noiseP, 'g', label='noise')
	plt.plot(chalP, 'b', label='channel')
	plt.legend(loc='center')
	plt.xlabel('Frame Index')
	plt.ylabel('Power')
	#plt.savefig('power_ch14_LOS_loc1.pdf')

	userCSI = csi[:,[cal_index,ue_index,4],:,:,:] #frame, user, LTS, bs ant, subcarrier  
	filename = 'ch1_' + loc_name + '.mat'
	varname = 'ch1_' + loc_name
	sio.savemat(filename, mdict={varname:userCSI})


    # channel 14
	HDF5_NAME = 'ch14_' + loc_name + '.hdf5'
	print(HDF5_NAME)
    
	frame = 100
	f = h5py.File(HDF5_NAME,'r')
	samps = f['Pilot_Samples']

	samps_per_user = f.attrs['samples_per_user']
	num_users = f.attrs['num_mob_ant']

	plt.figure(11,figsize=(25,6))
	plt.clf()
	for u in range(samps.shape[1]): # u is antenna element
		plt.plot(samps[frame,u,:,0])  #plot i/q samples of each antenna at

	csi,iq = samps2csi_large(f['Pilot_Samples'], f.attrs['num_mob_ant']+1, samps_per_user=samps_per_user) #
	userCSI = csi[:,:,0,:,:] #Frame, User, BS ant, Subcarrier
	#userCSI = np.mean(csi[:,:,:,:,:],2)

	frame_num = userCSI.shape[0]
	frame_rem = 10
	noiseP = np.empty((frame_num - frame_rem,))
	chalP = np.empty((frame_num - frame_rem,))
	corr = np.empty((frame_num - frame_rem,))
	#carr_index = 0
	chal1 = userCSI[frame_rem,ue_index,4:68,carr_index]
	for i in range(frame_num - frame_rem):
		noise = userCSI[frame_rem + i,4,4:68,carr_index]
		noiseP[i] = np.mean(np.square(np.abs(noise)))
		chal = userCSI[frame_rem + i,ue_index,4:68,carr_index]
		chalP[i] = np.mean(np.square(np.abs(chal)))
		corr[i] = np.absolute(np.correlate(chal,chal1)/(np.linalg.norm(chal) * np.linalg.norm(chal1)))


	plt.figure(12)
	plt.clf()
	plt.plot(corr)
	plt.xlabel('Frame Index')
	plt.ylabel('Correlation')
	#plt.savefig('cor_ch14_LOS_loc1.pdf')

	plt.figure(13)
	plt.clf()
	plt.plot(noiseP, 'g', label='noise')
	plt.plot(chalP, 'b', label='channel')
	plt.legend(loc='center')
	plt.xlabel('Frame Index')
	plt.ylabel('Power')
	#plt.savefig('power_ch14_LOS_loc1.pdf')

	userCSI = csi[:,[cal_index,ue_index,4],:,:,:] #frame, user, LTS, bs ant, subcarrier  
	filename = 'ch14_' + loc_name + '.mat'
	varname = 'ch14_' + loc_name
	sio.savemat(filename, mdict={varname:userCSI}) 
