#******************************************************************************
# 
# File   : phase_calibrator.py
# Authors:	Andrew Brooks (agb3 [at] rice.edu)
# License:	Copyright 2015, Rice Efficient Computing Group. All rights reserved.
#
#******************************************************************************


import numpy as np
import fractions

def lteZadoffChuSeq(R, N):
	""" 
	Generate a root Zadoff-Chu sequence of complex symbols.

	lteZadoffChuSeq(R, N) generates the Rth root Zadoff-Chu sequence of length N as per LTE specifications. The output SEQ is an N-length column vector of complex symbols.
	"""



	if N%2 == 0:
		raise Exception ("The sequence length must be an odd-valued positive integer scalar.")

	if fractions.gcd(N,R) != 1:
		raise Exception ("Greatest Common Denominator error")

	#m = (0:N-1).'
	m = np.arange(N).transpose()
	seq = np.exp( -1j * np.pi * R * m*(m+1) / N )

	return seq
