# -*- coding: utf-8 -*-
"""
Created on Sun Mar 13 07:22:54 2016

@author: Clayton Shepard
"""


from argoscc.channel_analysis import *
import glob
import os

def beamformedCap(userCSI, beamweights, noise_sc_bs, downlink=True):
	"""
		Calculate the per-user shannon capacity given a set of csi estimates and beamweights
		userCSI with (frame, sc, user, bs_ant), beamweights with (frame, sc, bs_ant, user), noise_sc_bs with (sc, bs)
	"""	
	sig_intf = np.square(np.abs(np.matmul(userCSI[:,:,:],beamweights[:,:,:])))
	#noise_sc_u = np.empty((userCSI.shape[0],userCSI.shape[1]),dtype='float32')	
	#for sc in range(userCSI.shape[1]):
	#	noise_sc_u[sc,:] = np.dot(np.square(np.abs(noise_sc_bs[sc,:])),np.square(np.abs(beamweights[sc,:,:]))) #noise is uncorrelated, and all we have is average power here (Evan wants to do it per frame, but I think that's a bad idea)
		#noise_sc_u[sc,:] = np.sum(np.square(np.abs(noise_sc_bs[sc,:])*np.square(np.abs(np.swapaxes(beamweights[sc,:,:],0,1) )),1)
	noise_sc_u = np.transpose(np.sum(np.square(noise_sc_bs)*np.square(np.abs(np.transpose(beamweights,(2,0,1)) )),2),(1,0))	
	sig_sc = np.diagonal(sig_intf,axis1=1,axis2=2)
	intf_sc = np.sum(sig_intf,axis=1+int(downlink)) - sig_sc  
	SINR = sig_sc/(noise_sc_u+intf_sc)
		
	cap_sc = np.log2(1+SINR)
	cap_u = np.mean(cap_sc,axis=0)	
	SINR_u = np.mean(SINR,axis=0)

	return cap_u, SINR_u

def findMobilityCorr(userCSI, threshold=.05, method='both'):
	"""
		userCSI with (frame, sc, bs_ant, lts)
		correlate the last n frames and determine if the current one changed from the average substantially.
		Note that the threshold for both avg and standard deviation should be roughly between 0.01 and 0.05
		Standard deviation is making sure that the channel has been stable for all samples, whereas avg is more sensitive to the current measurement being the same as the rest.
	"""
	userCSI = np.mean(userCSI,-1) #let's only look at the average for now.
	c = np.mean(np.abs(np.sum(np.conj(userCSI[0,:,:])*userCSI[1:,:,:],-1))/(np.sum(np.abs(userCSI[0,:,:])*np.abs(userCSI[1:,:,:]),-1)), 1) #correlate the last n CSI estimates, and average over subcarrier

	#return True	#always mobile, so always estimate channel
	#print(np.abs( c[-1]-np.mean(c) ) , np.std(c))
	a = np.abs( c[-1]-np.mean(c) ) > threshold
	s = np.std(c) > threshold
	if method is 'both':
		return a or s
	if method is 'avg':
		return a
	elif method is 'std':
		return s
	

def findMobilitySNR(zf_sinr, conj_sinr, last_csi, noise_sc_bs, threshold=3):
	"""
		last_csi with (sc,bs_ant,lts)
		determine if the user is mobile based on the achieved SNR.
		threshold is in dB.  .75 dB is roughly good for conj.  6 dB for same frame or 15 dB (or even more) is roughly good for zf.
	"""
	conj_beamweights = np.transpose(np.conj(last_csi[:,:,:,1]),(0,2,1))
	for sc in range(last_csi.shape[0]):
		zf_beamweights[sc,:,:] = np.linalg.pinv( last_csi[sc,:,:,1] ) 
	per_user_cap_conj,per_user_sinr_conj = beamformedCap(last_csi[:,:,:,0], conj_beamweights, noise_sc_bs)
	per_user_cap_zf,per_user_sinr_zf = beamformedCap(last_csi[:,:,:,0], zf_beamweights, noise_sc_bs)	
	
	zf_mob = 10*np.abs(np.log10(per_user_sinr_zf)-np.log10(zf_sinr)) > threshold
	conj_mob = 10*np.abs(np.log10(per_user_sinr_conj)-np.log10(conj_sinr)) > threshold
	
	return conj_mob, per_user_sinr_conj
	#return zf_mob, per_user_sinr_zf
	
#-----------------------------------------------------------------------------
# Main Function
#-----------------------------------------------------------------------------
if __name__ == '__main__':
	starttime = time.time()

	if len(sys.argv) >= 2:
		HDF5_NAME = str(sys.argv[1])
	else:
		HDF5_NAME = max(glob.iglob('logs/*.[Hh][Dd][Ff]5'), key=os.path.getctime)
	print(HDF5_NAME)
	
	h5log = h5py.File(HDF5_NAME,'r')	

	samps_per_user = h5log.attrs['samples_per_user']
	num_users = h5log.attrs['num_mob_ant']
	timestep = h5log.attrs['frame_length']/20e6			
	
	#compute CSI for each user and get a nice numpy array
	csi,iq = samps2csi(h5log['Pilot_Samples'][1000:6000:5], num_users+1, samps_per_user) #Returns csi with Frame, User, LTS (there are 2), BS ant, Subcarrier  #also, iq samples nicely chunked out, same dims, but subcarrier is sample.
	#[1500:2500]
	del iq
	#csi = csi[:,[0]] #ugh, can zf 1 user
	
	n = 0
	if n != 0:
		csi += np.random.normal(0, 0.5, csi.shape)*n*1.0j + np.random.normal(0, 0.5, csi.shape)*n

	csi_sw = np.transpose(csi,(0,4,1,3,2)) #hack to avoid for loop (matmul requires last two axes to be matrix) #frame, sc, user, bsant, lts
	noise = csi_sw[:,:,-1,:,:]  #noise is last set of data. #frame, sc, bsant, lts
	noise_sc_bs = np.mean(np.mean(np.abs(noise),0),-1) 	
	userCSI = csi_sw[:,:,:-1,:,:] #frame, sc, user, bsant, lts	
	
	num_frames_test_mob = 10  # how many frames to judge stability off of.
	use_next_frame = True	
	
	per_user_cap_conj = np.empty((userCSI.shape[0]-int(use_next_frame),userCSI.shape[2]),dtype='float32')
	per_user_cap_zf = np.empty((userCSI.shape[0]-int(use_next_frame),userCSI.shape[2]),dtype='float32')
	per_user_sinr_conj = np.empty((userCSI.shape[0]-int(use_next_frame),userCSI.shape[2]),dtype='float32')
	per_user_sinr_zf = np.empty((userCSI.shape[0]-int(use_next_frame),userCSI.shape[2]),dtype='float32')
	#last_good_csi = np.empty((userCSI.shape[0],userCSI.shape[2]),dtype='uint32')
	zf_beamweights = np.empty((userCSI.shape[1],userCSI.shape[3],userCSI.shape[2]),dtype='complex64') #sc, bsant, users
		
	per_user_cap_conj_nomab = np.empty((userCSI.shape[0]-int(use_next_frame),userCSI.shape[2]),dtype='float32')
	per_user_cap_zf_nomab = np.empty((userCSI.shape[0]-int(use_next_frame),userCSI.shape[2]),dtype='float32')
	per_user_sinr_conj_nomab = np.empty((userCSI.shape[0]-int(use_next_frame),userCSI.shape[2]),dtype='float32')
	per_user_sinr_zf_nomab = np.empty((userCSI.shape[0]-int(use_next_frame),userCSI.shape[2]),dtype='float32')
	#last_good_csi = np.empty((userCSI.shape[0],userCSI.shape[2]),dtype='uint32')
	zf_beamweights_nomab = np.empty((userCSI.shape[1],userCSI.shape[3],userCSI.shape[2]),dtype='complex64') #sc, bsant, users
	
	user_mob = np.empty((userCSI.shape[0]-int(use_next_frame),userCSI.shape[2]),dtype='bool')
	user_mob[0:num_frames_test_mob,:] = True  #first 10 frames are assumed to be mobile
	
	current_csi = np.empty(userCSI.shape[1:],dtype='complex64') #sc, users, bsant, lts
	current_csi[:,:,:,:] = userCSI[num_frames_test_mob-1,:,:,:,:]

	for frame in range(num_frames_test_mob):
		conj_beamweights = np.transpose(np.conj(userCSI[frame,:,:,:,1]),(0,2,1))
		for sc in range(userCSI.shape[1]):
			zf_beamweights[sc,:,:] = np.linalg.pinv( userCSI[frame,sc,:,:,1] ) 
		per_user_cap_conj[frame,:],per_user_sinr_conj[frame,:] = beamformedCap(userCSI[frame+int(use_next_frame),:,:,:,0], conj_beamweights, noise_sc_bs)
		per_user_cap_zf[frame,:],per_user_sinr_zf[frame,:] = beamformedCap(userCSI[frame+int(use_next_frame),:,:,:,0], zf_beamweights, noise_sc_bs)		
				
		conj_beamweights_nomab = np.transpose(np.conj(userCSI[frame,:,:,:,1]),(0,2,1))
		for sc in range(userCSI.shape[1]):
			zf_beamweights_nomab[sc,:,:] = np.linalg.pinv( userCSI[frame,sc,:,:,1] ) 
		per_user_cap_conj_nomab[frame,:],per_user_sinr_conj[frame,:] = beamformedCap(userCSI[frame+int(use_next_frame),:,:,:,0], conj_beamweights_nomab, noise_sc_bs)
		per_user_cap_zf_nomab[frame,:],per_user_sinr_zf[frame,:] = beamformedCap(userCSI[frame+int(use_next_frame),:,:,:,0], zf_beamweights_nomab, noise_sc_bs)			

	for frame in range(num_frames_test_mob,userCSI.shape[0]-int(use_next_frame)):
		#status read out
		if frame % 100 == 0:
			print(frame)
			
		#user_mob[frame,:] = findMobility(userCSI[frame-num_frames_test_mob:frame,:,:,:,:], current_csi, user_mob[frame-1,:])
	
		#if users are mobile, we get current csi, and check if they have become stationary using the correlation method
		#if not then we use the old csi and check if they have become mobile.
		temp_mob, sinr_rec = findMobilitySNR(per_user_sinr_zf[frame-1,:],per_user_sinr_conj[frame-1,:],current_csi[:,:,:,:], noise_sc_bs,threshold=2.5)  #predict snrs based on last known CSI and compare to acutal sinr from the previous frame.  this needs to be done before current_csi is updated.
		#print(temp_mob, sinr_rec, per_user_sinr_zf[frame-1,:])		
		for user in range(userCSI.shape[2]):
			if user_mob[frame-1,user]:
				current_csi[:,user,:,:] = userCSI[frame,:,user,:,:]
				if np.all(user_mob[frame-num_frames_test_mob:frame,user]): #we have to stay mobile long enough to collect n frames
					user_mob[frame,user] =	findMobilityCorr(userCSI[frame-num_frames_test_mob+1:frame+1,:,user,:,:],method='avg', threshold=.03)
				else:
					user_mob[frame,user] = True
			else:
				user_mob[frame,user] = temp_mob[user]
				
		conj_beamweights = np.transpose(np.conj(current_csi[:,:,:,1]),(0,2,1))
		for sc in range(userCSI.shape[1]):
			zf_beamweights[sc,:,:] = np.linalg.pinv( current_csi[sc,:,:,1] ) 
		per_user_cap_conj[frame,:],per_user_sinr_conj[frame,:] = beamformedCap(userCSI[frame+int(use_next_frame),:,:,:,0], conj_beamweights, noise_sc_bs)
		per_user_cap_zf[frame,:],per_user_sinr_zf[frame,:] = beamformedCap(userCSI[frame+int(use_next_frame),:,:,:,0], zf_beamweights, noise_sc_bs)			

				
		conj_beamweights_nomab = np.transpose(np.conj(userCSI[frame,:,:,:,1]),(0,2,1))
		for sc in range(userCSI.shape[1]):
			zf_beamweights_nomab[sc,:,:] = np.linalg.pinv( userCSI[frame,sc,:,:,1] ) 
		per_user_cap_conj_nomab[frame,:],per_user_sinr_conj[frame,:] = beamformedCap(userCSI[frame+int(use_next_frame),:,:,:,0], conj_beamweights_nomab, noise_sc_bs)
		per_user_cap_zf_nomab[frame,:],per_user_sinr_zf[frame,:] = beamformedCap(userCSI[frame+int(use_next_frame),:,:,:,0], zf_beamweights_nomab, noise_sc_bs)			

	if False: #use only first user
		per_user_cap_conj = per_user_cap_conj[:,[0]]
		per_user_cap_zf = per_user_cap_zf[:,[0]]
		user_mob = user_mob[:,[0]]
		per_user_cap_conj_nomab = per_user_cap_conj_nomab[:,[0]]
		per_user_cap_zf_nomab = per_user_cap_zf_nomab[:,[0]]
		num_users = 1
	
	print(np.mean(user_mob)) #channel estimation savings
	
	#total_cap_zf = np.empty((userCSI.shape[0]),dtype='float32')
	#total_cap_conj = np.empty((userCSI.shape[0]),dtype='float32')
	
	fixed_overhead = 4096 #beacon, tdd_guard, other control channel operations
	samples_per_pilot = 512 #pilot overhead
	percent_overhead = 48/80 #CP and unused/reference subcarriers (802.11 like)
	#percent_overhead = 1200/2192*30.72/20 #CP and unused/reference subcarriers (LTE like)
	framelength = .005*20e6
	zf_overhead = .001*20e6  #1ms for computing and distributing beamweights
	
	#num_users_sim = 32
	num_users_sim = num_users
	
	overhead = (1 - (np.sum(user_mob,1)*samples_per_pilot*num_users*num_users_sim/num_users+fixed_overhead)/framelength)*percent_overhead
	overhead_zf = (1 - (np.sum(user_mob,1)*samples_per_pilot*num_users*num_users_sim/num_users+fixed_overhead+zf_overhead)/framelength)*percent_overhead
		
	total_cap_conj = np.mean(np.sum(per_user_cap_conj, 1)*num_users_sim/num_users*overhead)
	total_cap_zf = np.mean(np.sum(per_user_cap_zf, 1)*overhead_zf)*num_users_sim/num_users
	print(total_cap_conj,total_cap_zf)
	print(np.mean(per_user_cap_conj),np.mean(per_user_cap_zf),np.mean(per_user_cap_conj_nomab),np.mean(per_user_cap_zf_nomab))
	
	results = {}
	perf_zf = []
	perf_conj = []
	for num_users_sim in range(1,65): #[4,8,16,32,64]:
		overhead = (1 - (np.sum(user_mob,1)*samples_per_pilot*num_users_sim/num_users+fixed_overhead)/framelength)*percent_overhead
		overhead_zf = (1 - (np.sum(user_mob,1)*samples_per_pilot*num_users_sim/num_users+fixed_overhead+zf_overhead)/framelength)*percent_overhead
		
		total_cap_conj_mab = np.mean(np.sum(per_user_cap_conj, 1)*num_users_sim/num_users*overhead)
		total_cap_zf_mab = np.mean(np.sum(per_user_cap_zf, 1)*overhead_zf)*num_users_sim/num_users
		#print(total_cap_conj,total_cap_zf)
		
		#no MAB
		overhead = (1 - (samples_per_pilot*num_users_sim+fixed_overhead)/framelength)*percent_overhead
		overhead_zf = (1 - (samples_per_pilot*num_users_sim+fixed_overhead+zf_overhead)/framelength)*percent_overhead
		
		total_cap_conj = np.mean(np.sum(per_user_cap_conj_nomab, 1)*num_users_sim/num_users*overhead)
		total_cap_zf = np.mean(np.sum(per_user_cap_zf_nomab, 1)*overhead_zf)*num_users_sim/num_users
		#print(total_cap_conj,total_cap_zf)
		results[num_users_sim] = [total_cap_conj_mab,total_cap_zf_mab,total_cap_conj,total_cap_zf]
		
		#print('%d users conj: %f zf: %f' % (num_users_sim, total_cap_conj_mab/total_cap_conj, total_cap_zf_mab/total_cap_zf))
		#print('(%d, %d)' % (num_users_sim,total_cap_conj_mab/total_cap_conj) )
		#print('(%d, %f)' % (num_users_sim,total_cap_zf_mab/total_cap_zf) )
		perf_zf.append(total_cap_zf_mab/total_cap_zf)
		perf_conj.append(total_cap_conj_mab/total_cap_conj)
	
	
	import scipy.io
	data = dict(perf_conj=perf_conj,perf_zf=perf_zf)
	scipy.io.savemat('logs/asilomar-perf_static.mat', data)
	'''
load asilomar-perf_mobile.mat
perf_conj_mob = perf_conj;
perf_zf_mob = perf_zf;
load asilomar-perf_static.mat
perf_conj_static = perf_conj;
perf_zf_static = perf_zf;
figure('Position', [100, 100, 335, 260]);
hold on
%plot(perf_conj_static)
plot(perf_zf_static)
%plot(perf_conj_mob)
plot(perf_zf_mob)
ylabel('Relative Performance')
xlabel('Number of Users')
%xlim([0,20])
ylim([.8,1.75])
legend('Static','Mobile')
%legend('Static Conj','Static ZF','Mobile Conj','Mobile ZF')
	'''
	
	#ZF
	plt.figure(0, figsize=(50,10))
	plt.plot(np.arange(per_user_cap_zf.shape[0])*timestep,per_user_cap_zf)
	#plt.ylim([0,2])		
	plt.xlabel('Time (s)')
	plt.ylabel('ZF Capacity (bps/Hz)')
	plt.show()
	
	#conj
	plt.figure(1, figsize=(50,10))
	plt.plot(np.arange(per_user_cap_conj.shape[0])*timestep,per_user_cap_conj)
	#plt.ylim([0,2])		
	plt.xlabel('Time (s)')
	plt.ylabel('Conj Capacity (bps/Hz)')
	plt.show()
	
	#ZF
	plt.figure(10, figsize=(50,10))
	plt.plot(np.arange(per_user_cap_zf.shape[0])*timestep,per_user_cap_zf_nomab)
	#plt.ylim([0,2])		
	plt.xlabel('Time (s)')
	plt.ylabel('ZF Capacity (bps/Hz)')
	plt.show()
	
	#conj
	plt.figure(11, figsize=(50,10))
	plt.plot(np.arange(per_user_cap_conj.shape[0])*timestep,per_user_cap_conj_nomab)
	#plt.ylim([0,2])		
	plt.xlabel('Time (s)')
	plt.ylabel('Conj Capacity (bps/Hz)')
	plt.show()
	
	#mobility
	plt.figure(2, figsize=(50,10))
	plt.plot(np.arange(user_mob.shape[0])*timestep,user_mob)
	plt.ylim([-.5,1.5])		
	plt.xlabel('Time (s)')
	plt.ylabel('User Mobile?')
	plt.show()	
