#******************************************************************************
# 
# File   : sd_cfg.py
# Authors:	Michael Tsehaie and Clay Shepard (cws [at] rice.edu)
# License:	Copyright 2015, Rice Efficient Computing Group. All rights reserved.
#
#******************************************************************************

import numpy as np
import struct

class sd_cfg():
	CMD_SD_INIT = 100
	CMD_SD_CONFIG = 110
	#CMD_SD_DEBUG_PRINT = 101
	CMD_SD_WRITE_BIN = 102

	sd_init = 100
	sd_reconfig = 110
	#sd_debug_print = 101
	sd_write_bitstream = 102


	def procCmd(self,cmdID, *args, mtu=1400):
		if type(cmdID) is str:
			cmdID = getattr(sd_cfg,cmdID.lower())
		if cmdID == sd_cfg.CMD_SD_CONFIG:
			slot = args[0]
			if ((slot<0) or (slot>7)):
				raise Exception('Invalid Slot - punting\n')
				return
		if cmdID == sd_cfg.CMD_SD_WRITE_BIN:
			#Input args:
	            # 1: Slot to write (0:7)
			# 2: .bin filename as string
			slot = args[0]
			if (slot == 0):
				raise Exception('Writing to slot 0 disabled on Argos!\n')
				return                     
			elif((slot < 1) or (slot > 7)):
				raise Exception('Invalid Slot - punting\n')
				return
				
			fd = open(args[1],"rb")
			fbin = struct.unpack('>2308111I',fd.read())
			if(len(fbin) != 2308111): 
				print('Invalid .bin file (wrong size)\n')
            #Pad bitstream to integral number of 512B blocks
			fbin = np.vstack((np.array(fbin).reshape(np.array(fbin).shape[0],1), np.zeros((-len(fbin)%128, 1)))) 
			num_blocks = len(fbin) / 128
			MaxPayload = mtu - (34 + 40);           
			blocks_per_pkt = np.floor(float(MaxPayload) / 512)
			num_pkts = int(np.ceil(num_blocks / blocks_per_pkt))
			args_list = []
			for p in range (num_pkts):
				argslist = []  
                #args:
					# 0: Byte offset into SD card to write
					# 1: Blocks in this pkt
					# 2: (512*blocks_in_pkt) bytes of data to write
				first_word_r = 1 + (p*blocks_per_pkt*128)
				first_byte_w = (131072*512) + (slot*512*32768) + (4*(first_word_r-1))
				argslist.append(int(first_byte_w))
				if(p == num_pkts-1):
					#Last packet - send trailing blocks
					blks = fbin[first_word_r-1 : len(fbin)-1]
					argslist.append(int(len(blks/128)))
				else:
					last_byte_r  = first_word_r -1 + blocks_per_pkt*128
					blks = fbin[first_word_r-1 : last_byte_r]
					argslist.append(int(blocks_per_pkt))
				blklist = blks.tolist()                                #convert from numpy array to list of list
				[argslist.append(int(listnum[0])) for listnum in blklist] #convert each float in list of list to int
				args_list.append(argslist)                              # add the ints to your arguments
			fd.close()
			args = args_list
		return cmdID, args
