# -*- coding: utf-8 -*-
"""
Created on Mon Mar  7 08:52:36 2016

@author: Clay Shepard
"""

from argoscc.channel_analysis import *
import glob
import os
#import sys
#import multiprocessing as mp
#from functools import partial

def calMobility(userCSI, num_samps=10, thresh=.05):
	"""
		userCSI with (frame, user, bs_ant, subcarrier)
		note, to perform for just one user, dimensionality has to be maintained, so slice like userCSI[:,[2],:,:]
	"""
	corr = np.empty((userCSI.shape[0]-num_samps,num_samps,userCSI.shape[1],userCSI.shape[3]),dtype='float32')
	sc_mean = np.empty((userCSI.shape[0]-num_samps,num_samps,userCSI.shape[1]),dtype='float32')
	diff = np.empty((userCSI.shape[0]-num_samps,num_samps,userCSI.shape[1]),dtype='float32')
	mob = np.empty((userCSI.shape[0]-num_samps,userCSI.shape[1]),dtype='bool')
	conjcsi = np.conj(userCSI[:,:,:,:]) #np.transpose(np.conj(userCSI[:,:,:,:]),(0,2,1,3))
	for frame in range(num_samps,userCSI.shape[0]):  #leave enough of the array
		for user in range(userCSI.shape[1]):
			for sc in range(userCSI.shape[3]):
				corr[frame-num_samps,:,user,sc] = np.abs(np.dot(userCSI[frame-num_samps:frame,user,:,sc],conjcsi[frame,user,:,sc]))/np.dot(np.abs(userCSI[frame-num_samps:frame,user,:,sc]),np.abs(conjcsi[frame,user,:,sc]))	
	sc_mean = np.mean(corr,3)
	for frame in range(userCSI.shape[0]-num_samps):	
		diff[frame,:,:] = sc_mean[frame,:,:] - np.mean(sc_mean,1)[frame,:]
	mob = np.abs(diff) > thresh
	
	return mob,diff


def calContCapacity(userCSI, noise, conj=True, downlink=False, offset=1):
	"""
	Apply a set of beamweights to a set of wideband user channels and calculate the shannon capacity of the resulting channel for every sample.
	"""
	#noise = csi[:,-1,:,:,:]
	#bws = np.transpose(np.conj(csi[frame,:4,0,:,:]),(1,0,2)) for conjugate or, better: bws = np.transpose(np.conj(np.mean(csi[frame,:4,:,:,:],1)),(1,0,2))
	#userCSI = np.mean(csi[:100,:4,:,:,:],2)
	#bws = np.empty((csi.shape[3],csi.shape[1]-1,csi.shape[4]),dtype='complex64')	
	#for sc in range(csi.shape[4]):
	#	bws[:,:,sc] = np.linalg.pinv( np.mean(csi[frame,:4,:,:,sc],1) ) #* np.linalg.norm(csi[frame,:4,0,:,sc]) #either this, or the noise power has to be scaled back accordingly


	noise_bs_sc = np.mean(np.mean(np.abs(noise),0),0)  #average over time and the two ltss
	#noise_bs_sc_scaled = noise_bs_sc * np.abs(np.sum(beamweights,1)) #should sum be outside abs?  probably not
	#noise_sc = np.sqrt(np.sum(noise_bs_sc_scaled,0)) # np.mean(noise_bs_sc_scaled,0)  #should be sum?  noise is not additive... this is where the gain is.  Maybe sqrt(of sum)?
	#sig_intf = np.empty((userCSI.shape[0]-1,userCSI.shape[1],userCSI.shape[1],userCSI.shape[3]),dtype='float32')
	noise_sc_u = np.empty((userCSI.shape[0]-offset,userCSI.shape[3],userCSI.shape[1]),dtype='float32')
	
	userCSI_sw = np.swapaxes(np.swapaxes(userCSI,3,2),2,1) #hack to avoid for loop (matmul requires last two axes to be matrix)
	if conj:	
		beamweights = np.transpose(np.conj(userCSI_sw),(0,1,3,2))
	else:
		beamweights = np.empty((userCSI_sw.shape[0],userCSI_sw.shape[1],userCSI_sw.shape[3],userCSI_sw.shape[2]),dtype='complex64')	
		for frame in range(userCSI_sw.shape[0]):		
			for sc in range(userCSI_sw.shape[1]):
				beamweights[frame,sc,:,:] = np.linalg.pinv( userCSI_sw[frame,sc,:,:] ) #* np.linalg.norm(csi[frame,:4,0,:,sc]) #either this, or the noise power has to be scaled back accordingly
	if offset > 0:
		beamweights = np.roll(beamweights, offset, axis=0) #delay offset samples
	
	sig_intf = np.square(np.abs(np.matmul(userCSI_sw[offset:,:,:,:],beamweights[offset:,:,:,:])))
	#noise_sc_u = np.square(np.abs(np.matmul(noise[1:,:,:,:],beamweights[:-1,:,:,:])))
	
	for sc in range(userCSI_sw.shape[1]):
		#sig_intf[:,:,:,sc] = np.square(np.abs(np.dot(userCSI[1:,:,:,sc],beamweights[:-1,:,:,sc])))  #TODO: can we get rid of the for loop?
		noise_sc_u[:,sc,:] = np.dot(np.square(noise_bs_sc[:,sc]),np.square(np.abs(beamweights[offset:,sc,:,:]))) #noise is uncorrelated, and all we have is average power here (Evan wants to do it per frame, but I think that's a bad idea)
		#noise_sc_u[:,sc] = np.square(np.abs(np.dot(noise_bs_sc[:,sc],beamweights[:,:,sc]))) #hrm, this is per Evan, but I don't think it's right... it should square then sum...
	
	#noise_sc_u *= 4 #fudge factor since our noise doesn't include a lot of noise sources.  this should probably be justified/measured or removed
	#we could increase interference a bit too, maybe by 1.3 or so
	#return sig_intf
	sig_sc = np.diagonal(sig_intf,axis1=2,axis2=3)
	#sig_sc = np.swapaxes(sig_sc,1,2)
	intf_sc = np.sum(sig_intf,axis=2+int(downlink)) - sig_sc  
	SINR = sig_sc/(noise_sc_u+intf_sc)
		
	cap_sc = np.log2(1+SINR)
	cap_u = np.mean(cap_sc,axis=1)
	cap_total = np.sum(cap_u,axis=1)
	
	#noise_bs_sc_scaled_u = np.swapaxes(noise_bs_sc * np.abs(np.swapaxes(beamweights,0,1)),0,1)
	#noise_sc_u = np.sqrt(np.sum(noise_bs_sc_scaled_u,0)) #np.mean(noise_bs_sc_scaled_u,0)	#should me sum?
	SNR = sig_sc/noise_sc_u	 #TODO, this noise is actually way too much (by roughly a factor of the number of users)
	cap_su_sc = np.log2(1+SNR)
	cap_su_u = np.mean(cap_su_sc,axis=1)
	
	return cap_total,cap_u,cap_sc,SINR,cap_su_sc,cap_su_u,SNR



# Long exposure version. Edited by abeer
def le_samps2csi(samps, num_users, samps_per_user=224, fft_size=64):
	"""Returns csi with Frame, User, LTS1, LTS2, BS, Subcarrier"""
	offset = 0
	# lts_iq = samps[frame,0,user*samps_per_user:(user+1)*samps_per_user,0]*1.+samps[frame,0,user*samps_per_user:(user+1)*samps_per_user,1]*1j
	# lts_iq /= 2**15	
	
	# offset = lts.findLTS(lts_iq)+32
	chunkstart = time.time()
	usersamps = np.reshape(samps, (samps.shape[0], samps.shape[1], num_users, samps_per_user, 2))
	# usersamps orderd by (frame, bs antenna, user, samples, real/imag)
	
	iq = np.empty((samps.shape[0],samps.shape[1], num_users, 2, fft_size),dtype='complex64')
	# iq orderd by (frame, bs antenna, user, real/imag, samps per fft)
	
	for i in range(2): #2 seperate estimates
		iq[:,:,:,i,:] = (usersamps[:,:,:,offset+i*fft_size:offset+(i+1)*fft_size,0]+usersamps[:,:,:,offset+i*fft_size:offset+(i+1)*fft_size,1]*1j)*2**-15
	
	iq = iq.swapaxes(1,2)  #this is trickery to keep numpy vectorized (fast), then keep the axis in the order we want
	iq = iq.swapaxes(2,3)
	fftstart = time.time()
	csi = np.fft.fftshift(np.fft.fft(iq, fft_size, 4),4)*lts.lts_freq
	endtime = time.time()
	print("chunk time: %f fft time: %f" % (fftstart - chunkstart, endtime - fftstart))
	csi = np.delete(csi,[0,1,2,3,4,5,32,59,60,61,62,63],4) #remove zero subcarriers
	return csi,iq	


def le_calCapacity(userCSI, noise, beamweights, downlink=False):
	"""
	Apply a set of beamweights to a set of wideband user channels and calculate the shannon capacity of the resulting channel for every sample.
	"""

	noise_bs_sc = np.mean(np.mean(np.abs(noise),0),0)  # Average over time and the two ltss
	# ordered by (bs antenna, subcarrier)
	
	# userCSI ordered by (frame, user, BS ant, Subcarrier)
	num_frames = userCSI.shape[0]
	num_users = userCSI.shape[1]
	num_subcarriers = userCSI.shape[3]
	
	sig_intf = np.empty((num_frames, num_users, num_users, num_subcarriers),dtype='float32')
	noise_sc_u = np.empty((num_users, num_subcarriers),dtype='float32')
	for sc in range(num_subcarriers):
		sig_intf[:,:,:,sc] = np.square(np.abs(np.dot(userCSI[:,:,:,sc], beamweights[:,:,sc])))
		noise_sc_u[:,sc] = np.dot(np.square(noise_bs_sc[:,sc]),np.square(np.abs(beamweights[:,:,sc])))
		# noise is uncorrelated, and all we have is average power here (Evan wants to do it per frame, but I think that's a bad idea)
		
	sig_sc = np.diagonal(sig_intf,axis1=1,axis2=2)
	sig_sc = np.swapaxes(sig_sc,1,2)
	
	# intf_sc = np.sum(sig_intf,axis=1+int(downlink)) - sig_sc  
	# SINR = sig_sc/(noise_sc_u+intf_sc)

	SINR = sig_sc/noise_sc_u
	cap_sc = np.log2(1+SINR)

	# Average capacity over all subcarriers
	cap_u = np.mean(cap_sc,axis=2)

	# Total capacity over all users
	cap_total = np.sum(cap_u,axis=1)
		
	return cap_total, cap_u

def getCalibratedCSI(f):
	samps_per_user = f.attrs['samples_per_user']
	num_users = f.attrs['num_mob_ant']
	csi,iq = samps2csi(f['Pilot_Samples'], num_users+1, samps_per_user)
	csi_front,iq_front = samps2csi(f['AoA_Front']['Pilot_Samples'], num_users+1,samps_per_user)
	#csi_calib = csi/csi_front[csi_front.shape[0]//2,0,0,:,:]
	calib = np.mean(np.mean(csi_front[csi_front.shape[0]//2-5:csi_front.shape[0]//2+5,0,:,:,:], axis=0),axis=0)
	csi_calib = csi/calib
	
	front_calib = csi_front/calib
	"""
	plt.figure(18,figsize=(10,6))
	plt.clf()
	plt.xlabel('CSI Angle of Front Calibrated')
	plt.ylabel('Subcarrier')
	plt.plot(np.angle(front_calib[40,0,0,0:9,0]))	
	
	plt.figure(19,figsize=(10,6))
	plt.clf()
	plt.xlabel('Antenna')
	plt.ylabel('Angle')
	plt.plot(np.angle(front_calib[41,0,0,0:9,0]))	

	
	
	plt.figure(17,figsize=(10,6))
	plt.clf()
	plt.xlabel('CSI Angle of Calibration')
	plt.ylabel('Subcarrier')
	plt.plot(np.angle(calib[0,:]))	
	
	plt.figure(18,figsize=(10,6))
	plt.clf()
	plt.xlabel('CSI Angle of Front Calibrated')
	plt.ylabel('Subcarrier')
	plt.plot(np.angle(front_calib[30:40,0,0,0,:]).transpose())	
	"""
	
	return csi_calib,iq
	
	
	

'''
def calCoherence_helper_old(corr,userCSI,conjcsi,frame,user,num_samps,sc):
	corr[frame,:,user,sc] = np.abs(np.dot(userCSI[frame:frame+num_samps,user,:,sc],conjcsi[frame,user,:,sc]))/np.dot(np.abs(userCSI[frame:frame+num_samps,user,:,sc]),np.abs(conjcsi[frame,user,:,sc]))	
	
def calCoherence_helper(userCSI,conjcsi,sc):
	return np.abs(np.dot(userCSI[:,:,sc],conjcsi[:,sc]))/np.dot(np.abs(userCSI[:,:,sc]),np.abs(conjcsi[:,sc]))	
	


def calCoherence_parallel(userCSI,interval=5,timestep=.01):
	"""
		userCSI with (frame, user, bs_ant, subcarrier)
		note, to perform for just one user, dimensionality has to be maintained, so slice like userCSI[:,[2],:,:]
	"""
	p = mp.Pool(mp.cpu_count())
	
	num_samps = int(interval//timestep)
	corr = np.empty((userCSI.shape[0]-num_samps,num_samps,userCSI.shape[1],userCSI.shape[3]),dtype='float32')
	conjcsi = np.conj(userCSI[:,:,:,:]) #np.transpose(np.conj(userCSI[:,:,:,:]),(0,2,1,3))
	for frame in range(userCSI.shape[0]-num_samps):	 #leave enough of the array
		for user in range(userCSI.shape[1]):
			#for sc in range(userCSI.shape[3]):
			#calCoherence_worker = partial(calCoherence_helper, corr,userCSI,conjcsi,frame,user,num_samps)
			calCoherence_worker = partial(calCoherence_helper, userCSI[frame:frame+num_samps,user,:,:],conjcsi[frame,user,:,:])
			res = p.map(calCoherence_worker, range(userCSI.shape[3]))
			for sc,vals in enumerate(res):
				corr[frame,:,user,sc] = vals
	
	return corr
'''





















#  ***********Example Code*****************
if __name__ == '__main__':
	starttime = time.time()

	if len(sys.argv) >= 2:
		HDF5_NAME = str(sys.argv[1])
	else:
		HDF5_NAME = max(glob.iglob('logs/*.[Hh][Dd][Ff]5'), key=os.path.getctime)
	print(HDF5_NAME)

	frame = 50
	zoom = 0
	
	h5log = h5py.File(HDF5_NAME,'r')	

	samps_per_user = h5log.attrs['samples_per_user']
	num_users = h5log.attrs['num_mob_ant']
	timestep = h5log.attrs['frame_length']/20e6			
	
	#compute CSI for each user and get a nice numpy array
	csi,iq = samps2csi(h5log['Pilot_Samples'], num_users+1, samps_per_user) #Returns csi with Frame, User, LTS (there are 2), BS ant, Subcarrier  #also, iq samples nicely chunked out, same dims, but subcarrier is sample.
	#csi,iq = samps2csi(h5log['Pilot_Samples'][:100,0:8,:,:], num_users+1, samps_per_user) #Returns csi with Frame, User, LTS (there are 2), BS ant, Subcarrier  #also, iq samples nicely chunked out, same dims, but subcarrier is sample.
	#csi,iq = samps2csi(h5log['Pilot_Samples'][:1000,:,:,:], num_users+1, samps_per_user) #Returns csi with Frame, User, LTS (there are 2), BS ant, Subcarrier  #also, iq samples nicely chunked out, same dims, but subcarrier is sample.
	if zoom > 0:  #zoom in too look at behavior around peak (and reduce processing time)
		csi = csi[frame-zoom:frame+zoom,:,:,:,:]
		frame = zoom  #recenter the plots (otherwise it errors)		


	n = 0 #.5
	if n != 0:
		csi += np.random.normal(0, 0.5, csi.shape)*n*1.0j + np.random.normal(0, 0.5, csi.shape)*n
		
	noise = csi[:,-1,:,:,:]  #noise is last set of data.
	userCSI = np.mean(csi[:,:num_users,:,:,:],2) #don't include noise, average over both LTSs
	#sys.exit(0)
	noise_sub = np.abs(csi[:,:num_users,0,:,:]-csi[:,:num_users,1,:,:])
	
	noise_bs_sc = np.mean(np.mean(np.abs(noise),0),0)
	isnr = noise_bs_sc / np.abs(userCSI)
	isnr_time = np.mean(np.mean(isnr,2),2)


	#noise_bs_sc = np.mean(np.mean(np.abs(noise_sub),0),0)
	isnr = noise_sub / np.abs(userCSI)
	isnr_sub_time = np.mean(np.mean(isnr,2),2)
	
	
	#import scipy.io
	#scipy.io.savemat('logs/USERcsi-logname_%s.mat' % h5log.filename, userCSI)

	#example lts find:
	user = 0
	#so, this is pretty ugly, but we want all the samples (not just those chunked from samps2csi), so we not only convert ints to the complex floats, but also have to figure out where to chunk the user from.
	lts_iq = h5log['Pilot_Samples'][frame,0,user*samps_per_user:(user+1)*samps_per_user,0]*1.+h5log['Pilot_Samples'][frame,0,user*samps_per_user:(user+1)*samps_per_user,1]*1j
	lts_iq /= 2**15		
	offset = lts.findLTS(lts_iq)+32  #Andrew wrote this, but I don't really like the way he did the convolve method...  works well enough for high SNRs.
	print("LTS offset for user %d, frame %d: %d" % (user, frame, offset) )


	#compute beamweights based on the specified frame.
	conjbws = np.transpose(np.conj(userCSI[frame,:,:,:]),(1,0,2))		
	zfbws = np.empty((userCSI.shape[2],userCSI.shape[1],userCSI.shape[3]),dtype='complex64')	
	for sc in range(userCSI.shape[3]):
		zfbws[:,:,sc] = np.linalg.pinv(userCSI[frame,:,:,sc])
	
	downlink = True
	#calculate capacity based on these weights
	#these return total capacity, per-user capacity, per-user/per-subcarrier capacity, SINR, single-user capacity(no inter-user interference), and SNR
	conj = calCapacity(userCSI, noise, conjbws, downlink=downlink) #conjcap_total,conjcap_u,conjcap_sc,conjSINR,conjcap_su_sc,conjcap_su_u,conjSNR
	zf = calCapacity(userCSI, noise, zfbws, downlink=downlink) #zfcap_total,zfcap_u,zfcap_sc,zfSINR,zfcap_su_sc,zfcap_su_u,zfSNR 
	
	corr_total, sig_sc = calCorr(userCSI, conjbws)
	
	
	conj_cont = calContCapacity(userCSI, noise, conj=True, downlink=downlink)
	conj_cont_nodelay = calContCapacity(userCSI, noise, conj=True, downlink=downlink, offset=0)
	conj_cont_nodelay_samelts = calContCapacitySameLTS(csi, conj=True, downlink=downlink, offset=0)
	zf_cont_nodelay = calContCapacity(userCSI, noise, conj=False, downlink=downlink, offset=0)
	zf_cont = calContCapacity(userCSI, noise, conj=False, downlink=downlink)
	zf_cont_nodelay_samelts = calContCapacitySameLTS(csi, conj=False, downlink=downlink, offset=0)
	
	pl = 0
	
	
	#Multiuser Conjugate
	plt.figure(1000*pl, figsize=(50,10))
	plt.plot(np.arange(0,csi.shape[0]*timestep,timestep)[:csi.shape[0]],conj[1])
	#plt.ylim([0,2])		
	plt.xlabel('Time (s)')
	plt.ylabel('Per User Capacity Conj (bps/Hz)')
	plt.show()
	
	#Multiuser Zeroforcing
	plt.figure(1000*pl+1, figsize=(50,10))
	plt.plot(np.arange(0,csi.shape[0]*timestep,timestep)[:csi.shape[0]],zf[1])
	#plt.ylim([0,2])			
	plt.xlabel('Time (s)')
	plt.ylabel('Per User Capacity ZF (bps/Hz)')		
	plt.show()
	
	#Single user (but show all users)
	plt.figure(1000*pl+2, figsize=(50,10))
	plt.plot(np.arange(0,csi.shape[0]*timestep,timestep)[:csi.shape[0]],conj[-2])
	#plt.ylim([0,2])		
	plt.xlabel('Time (s)')
	plt.ylabel('SUBF Capacity Conj (bps/Hz)')
	plt.show()
	
	#Multiuser Correlation
	plt.figure(1000*pl+3, figsize=(50,10))
	plt.plot(np.arange(0,csi.shape[0]*timestep,timestep)[:csi.shape[0]],corr_total)
	#plt.ylim([0,2])		
	plt.xlabel('Time (s)')
	plt.ylabel('Per User Correlation')
	plt.show()

	#noise
	plt.figure(1000*pl+4, figsize=(50,10))
	plt.plot(np.arange(0,csi.shape[0]*timestep,timestep)[:csi.shape[0]],isnr_time)
	#plt.ylim([0,2])		
	plt.xlabel('Time (s)')
	plt.ylabel('Inverse SNR')
	plt.show()
	
	#noise
	plt.figure(1000*pl+5, figsize=(50,10))
	plt.plot(np.arange(0,csi.shape[0]*timestep,timestep)[:csi.shape[0]],1-.075*np.square(isnr_sub_time))
	#plt.ylim([0,2])		
	plt.xlabel('Time (s)')
	plt.ylabel('Inverse SNR')
	plt.show()
	
	mob,diff = calMobility(userCSI, thresh=.02)
	#mobility
	plt.figure(1000*pl+6, figsize=(50,10))
	plt.plot(np.arange(0,mob.shape[0]*timestep,timestep)[:mob.shape[0]],mob[:,9,:]) #(mob[:,9,:] + mob[:,8,:] + mob[:,7,:]) )
	plt.ylim([-.5,1.5])	
	plt.xlabel('Time (s)')
	plt.ylabel('Stationary')
	plt.show()
	
	#mobility
	plt.figure(1000*pl+9, figsize=(50,10))
	plt.plot(np.arange(0,mob.shape[0]*timestep,timestep)[:mob.shape[0]-1],np.logical_and(mob[:-1,9,:],mob[1:,9,:])) #(mob[:,9,:] + mob[:,8,:] + mob[:,7,:]) )
	plt.ylim([-.5,1.5])	
	plt.xlabel('Time (s)')
	plt.ylabel('Stationary')
	plt.show()
	
	#mobility
	plt.figure(1000*pl+10, figsize=(50,10))
	plt.plot(np.arange(0,mob.shape[0]*timestep,timestep)[:mob.shape[0]-2],np.logical_and(np.logical_and(mob[:-2,9,:],mob[2:,9,:]),mob[1:-1,9,:])) #(mob[:,9,:] + mob[:,8,:] + mob[:,7,:]) )
	plt.ylim([-.5,1.5])	
	plt.xlabel('Time (s)')
	plt.ylabel('Stationary')
	plt.show()	
	
	#mobility
	plt.figure(1000*pl+7, figsize=(50,10))
	plt.plot(np.arange(0,mob.shape[0]*timestep,timestep)[:mob.shape[0]],np.all(mob,2))
	plt.ylim([-.5,1.5])		
	plt.xlabel('Time (s)')
	plt.ylabel('Stationary')
	plt.show()

	#mobility
	plt.figure(1000*pl+8, figsize=(50,10))
	plt.plot(np.arange(0,mob.shape[0]*timestep,timestep)[:mob.shape[0]], diff[:,9,:] )
	#plt.ylim([0,2])		
	plt.xlabel('Time (s)')
	plt.ylabel('Mobility Thresh')
	plt.show()
	
	pl += 1
	
	#Multiuser Conjugate
	plt.figure(1000*pl, figsize=(50,10))
	plt.plot(np.arange(0,csi.shape[0]*timestep,timestep)[:csi.shape[0]-1],conj_cont[1])
	#plt.ylim([0,2])		
	plt.xlabel('Time (s)')
	plt.ylabel('Per User Capacity Conj Cont (bps/Hz)')
	plt.show()
	
	#Multiuser Zeroforcing
	plt.figure(1000*pl+1, figsize=(50,10))
	plt.plot(np.arange(0,csi.shape[0]*timestep,timestep)[:csi.shape[0]-1],zf_cont[1])
	#plt.ylim([0,2])			
	plt.xlabel('Time (s)')
	plt.ylabel('Per User Capacity ZF Cont (bps/Hz)')		
	plt.show()
	
	#Single user (but show all users)
	plt.figure(1000*pl+2, figsize=(50,10))
	plt.plot(np.arange(0,csi.shape[0]*timestep,timestep)[:csi.shape[0]-1],conj_cont[-2])
	#plt.ylim([0,2])		
	plt.xlabel('Time (s)')
	plt.ylabel('SUBF Capacity Conj Cont (bps/Hz)')
	plt.show()

	pl += 1
	
	#Multiuser Conjugate
	plt.figure(1000*pl, figsize=(50,10))
	plt.plot(np.arange(0,csi.shape[0]*timestep,timestep)[:csi.shape[0]],conj_cont_nodelay[1])
	#plt.ylim([0,2])		
	plt.xlabel('Time (s)')
	plt.ylabel('Per User Capacity Conj Cont No Delay (bps/Hz)')
	plt.show()
	
	#Multiuser Zeroforcing
	plt.figure(1000*pl+1, figsize=(50,10))
	plt.plot(np.arange(0,csi.shape[0]*timestep,timestep)[:csi.shape[0]],zf_cont_nodelay[1])
	#plt.ylim([0,2])			
	plt.xlabel('Time (s)')
	plt.ylabel('Per User Capacity ZF Cont No Delay(bps/Hz)')		
	plt.show()
	
	#Single user (but show all users)
	plt.figure(1000*pl+2, figsize=(50,10))
	plt.plot(np.arange(0,csi.shape[0]*timestep,timestep)[:csi.shape[0]],conj_cont_nodelay[-2])
	#plt.ylim([0,2])		
	plt.xlabel('Time (s)')
	plt.ylabel('SUBF Capacity Conj Cont No Delay(bps/Hz)')
	plt.show()

	pl += 1
	
	#Multiuser Conjugate
	plt.figure(1000*pl, figsize=(50,10))
	plt.plot(np.arange(0,csi.shape[0]*timestep,timestep)[:csi.shape[0]],conj_cont_nodelay_samelts[1])
	#plt.ylim([0,2])		
	plt.xlabel('Time (s)')
	plt.ylabel('Per User Capacity Conj Cont No Delay Same LTS (bps/Hz)')
	plt.show()
	
	#Multiuser Zeroforcing
	plt.figure(1000*pl+1, figsize=(50,10))
	plt.plot(np.arange(0,csi.shape[0]*timestep,timestep)[:csi.shape[0]],zf_cont_nodelay_samelts[1])
	#plt.ylim([0,2])			
	plt.xlabel('Time (s)')
	plt.ylabel('Per User Capacity ZF Cont No Delay Same LTS (bps/Hz)')		
	plt.show()
	
	#Single user (but show all users)
	plt.figure(1000*pl+2, figsize=(50,10))
	plt.plot(np.arange(0,csi.shape[0]*timestep,timestep)[:csi.shape[0]],conj_cont_nodelay_samelts[-2])
	#plt.ylim([0,2])		
	plt.xlabel('Time (s)')
	plt.ylabel('SUBF Capacity Conj Cont No Delay Same LTS (bps/Hz)')
	plt.show()
