#******************************************************************************
# 
# File   : wl_baseband.py
# Authors:	Clay Shepard (cws [at] rice.edu)
# License:	Copyright 2015, Rice Efficient Computing Group. All rights reserved.
#
#******************************************************************************

import numpy as np
import struct
class wl_baseband():

	GRPID = 0x30
	GRP = 'baseband'
	CMD_TX_DELAY = 1
	CMD_TX_LENGTH = 2
	CMD_TX_MODE = 3
	CMD_TX_BUFF_EN = 4
	CMD_RX_BUFF_EN = 5
	CMD_TX_RX_BUFF_DIS = 6
	CMD_WRITE_IQ = 7
	CMD_READ_IQ = 8
	CMD_READ_RSSI = 9
	CMD_RX_LENGTH = 10
	CMD_AGC_STATE = 30
	CMD_AGC_THRESH = 31
	CMD_AGC_TARGET = 32
	CMD_AGC_NOISE_EST = 33
	CMD_AGC_DCO_EN_DIS = 34
	CMD_AGC_TRIG_DELAY = 35
	CMD_AGC_RESET = 36
	CMD_AGC_DONE_ADDR = 37
	CMD_BEAM_WEIGHTS = 100

	#user friendly
	tx_delay = 1
	tx_length = 2
	tx_mode = 3
	tx_buff_en = 4
	rx_buff_end = 5
	tx_rx_buff_dis = 6
	write_iq = 7
	read_iq = 8
	read_rssi = 9
	rx_length = 10
	agc_state = 30
	agc_thresh = 31
	agc_target = 32
	agc_noise_est = 33
	agc_dco_en_dis = 34
	agc_trig_delay = 35
	agc_reset = 36
	agc_done_addr = 37
	beam_weights = 100
	
	wl_baseband_cmds = {'tx_delay' : 1,
		'tx_length' : 2,
		'tx_mode' : 3,
		'tx_buff_en' : 4,
		'rx_buff_end' : 5,
		'tx_rx_buff_dis' : 6,
		'write_iq' : 7,
		'read_iq' : 8,
		'read_rssi' : 9,
		'tx_length' : 10,
		'agc_state' : 30,
		'agc_thresh' : 31,
		'agc_target' : 32,
		'agc_noise_est' : 33,
		'agc_dco_en_dis' : 34,
		'agc_trig_delay' : 35,
		'agc_reset' : 36,
		'agc_done_addr' : 37,
		'beam_weights' : 100
	}
	
	
#	def procCmd(self, cmdID, args=[]):
#		if type(cmdID) is str:
#			cmdID = getattr(self, cmdID.lower()) #wl_baseband_cmds[cmdID.lower()]
#		if cmdID == tx_delay:
#			if len(args) !== 0:
#				delay = args[0]
#				if delay < 0 or delay >= self.txIQLen:
#					print ("error")
#		
#		elif cmdID == tx_length:
#			if len(args) != 0:
#				length = args[0]
#				if length <1 or length > self.txIQLen:
#					print ("error")
#
#		elif cmdID == rx_length:
#			if len(args) !=0:
#				length = args[0]
#				if length <1 or length > self.txIQLen:
#					print ("error")
#
#		elif cmdID == write_iq:
#			samps = args[0]
#			numsamps = len(samps)
#			num_interfaces = self.num_interfaces
#			if num_interfaces != len(samps[0]):
#				print ("error")
#			if len(args) == 2:
#				offset = args[1]
#				if len(offset) = 1:
#					offset = np.array(0,np.ones(1,num_interfaces))
#			else:
#				offset = np.zeros(1,num_interfaces)
#			if max(offset) + len(samps) > obj.txIQLen:
#				print ("error")
#			samp_I_fi = [np.int(samp.real*2**15) for samp in samps]
#			samp_Q_fi = [np.int(samp.imag*2**15) for samp in samps]
#			#define mechanism to write
#			
#		elif cmdID == read_iq:
#			if len(args) == 0:
#				offset = 0
#				numamp = self.rxIQLen
#			elif len(args) == 2:
#				offset = args[0]
#				numSamp = args[1]
#			else:
#				print ("error")
#			if (offset + numSamp) > self.rxIQLen:
#				print ("error")
#			# define rxsamples here	
#		
#		elif cmdID == agc_state:
#			k = 1
#			gains = np.uint16(ret(2*(k-1)+1)&0x000000FF)
#			out = np.uint16(gains&0x03)
#			out = np.uint16(gains >> 2)
#			out = np.uint16(ret(2*(k-1)+2))
#
#		elif cmdID == agc_thresh:
#			if len(args) != 0:
#				thresh1 = args[0]
#				thresh2 = args[1]
#				thresh3 = args[2]
#			#put test cases here	
#			
#			if (thresh1 >= 0):
#				thresholds = np.uint32(thresh1)
#			else:
#				thresholds = np.uint32(256-(abs(thresh1))
#			
#			if (thresh2 >= 0):
#				thresholds = thresholds + np.uint32(thresh2) << 8
#			else:
#				thresholds = thresholds + np.uint32(256-(abs(thresh2))) << 8
#			if (thresh3 >=0):
#				thresholds = thresholds + np.uint32(thresh2) << 16
#			else:
#				thresholds = thresholds + np.uint32(256-(abs(thresh2))) << 16
#
#		elif cmdID == agc_target:
#			target = args[0]
#			#one test condition here
#			if target < 0:
#				target = 65536-abs(target)
#			
#		elif cmdID == agc_noise_est:
#			noiseEst = args[0]
#			#one test condition here
#			if noiseEst < 0:
#				noiseEst = 65536 - abs(target)
#				
#		elif cmdID == agc_dco_en_dis:
#			args = [1 if args[0] is  True else 0]
#
#		elif cmdID == agc_trig_delay:
#			trigDelay = args[0]
#			if trigDelay < 0 or trigDelay > 2**9 -1:
#				print ("error")
#			args = trigDelay
#
#		elif cmdID == beam_weights:
#			if len(args) != 0:
#				samp_I_fi = [np.int16(data.real)*2**15 for data in args[0]]
#				samp_I_fi = [np.int16(data.imag)*2**15 for data in args[0]]
#				
#		return cmdarg, arg,flag
		
	def procReply(self, cmdID = None, data = None):
		cmdArgs = struct.unpack(">%dI" % ( (len(data)-22)//4 ), data[22:])
		return cmdArgs
