import os.path
import subprocess
from setuptools import setup, find_packages
from setuptools.command.install import install

with open(os.path.join(os.path.dirname(__file__), 'README.rst')) as f:
	readme = f.read()

with open(os.path.join(os.path.dirname(__file__), 'LICENSE.txt')) as f:
	license = f.read()



class ArgosCCInstall(install):
	def run(self):
		subprocess.call(['make', '-C', 'argoscc/C/'])   #http://stackoverflow.com/questions/24062830/best-way-to-package-a-python-library-that-includes-a-c-shared-library  #change this to match the location of the makefile
		subprocess.call(['make', 'install-wheel', '-C', 'argoscc/C/'])	
		install.run(self)


setup(
	name = 'ArgosCC',
	version = '0.90',

	description = 'Argos Central Controller',
	long_description = readme,
	url = 'http://projectargos.org',

	author = 'Clayton Shepard',
	author_email = 'cws@rice.edu',

	license = license,
	platforms = ["Windows", "Linux", "Solaris", "Mac OS-X", "Unix"],
	classifiers =[
		'Development Status :: 4 - Beta',
		'Intended Audience :: Science/Research',
		'Intended Audience :: Developers',
		'Topic :: Software Development :: Build Tools',
		'License :: OSI Approved :: MIT License',
		'Programming Language :: Python :: 3.4',
		'Programming Language :: Python :: 3.5',
		'Programming Language :: Python :: 3.6',

	],
	keywords='Argos massive MIMO wireless basestation', 
	packages = find_packages(),  
		include_package_data=True,
		setup_requires = ['numpy>=1.10.4'],
	install_requires = [ #'numpy',  #>=1.10.4
		'scipy>=0.14.1',  
		'netifaces',
		#'cython',
		'h5py>=2.2.1',   
		#'pandas>=0.17.1', 
		'matplotlib>=1.4.2', 
	],
	#Use our install class, so it build the library (and only does on install)
	cmdclass = {'install' : ArgosCCInstall},
)
	

